# @file
# Makefile containing rules for building lwe source code and demo tests.

#
# Copyright (c) 2008 - 2010 Freescale Semiconductor, Inc.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

#
# The env var $CROSS_COMPILE should be set to powerpc-linux-gnu-
#

DTC := $(SYSROOT)/bin/dtc
LIBFDT_INC := $(SYSROOT)/include
HV_DIR := $(TOP_DIR)lwe/src/hv
HV_INC := $(TOP_DIR)lwe/inc/hv
HV_BIN := bin/hv
HV_DEP_H := hv_dep.h
LDFLAGS = -L$(TOP_DIR)lwe/src/bin -L$(TOP_DIR)lwe/images -L$(SYSROOT)/lib
LIBC_INC=-I$(SYSROOT)/include/libc

CC ?= gcc
AS ?= as
CPU ?= e500mc
CROSS_COMPILE ?= powerpc-linux-gnu-

ifneq ($(CPU),)
CC_CPU_FLAGS = -mcpu=$(CPU) -t$(CPU)
AS_CPU_FLAGS = -Wa,-m$(CPU)
endif

LIBC_CC_OPTS=-D_GCCPORT_ \
  -D__POWERPC__ \
  -D__PPC_EABI__ \
  -D__MWERKS__ \
  -include msl_c_directives_to_gcc.h \
  -fno-builtin \
  -mbig-endian

LWE_BIN := bin/lwe
LWE_DIR := $(TOP_DIR)lwe
LIB_INC := $(LWE_DIR)/inc
LIB_INC_PUB := $(LWE_DIR)/inc/public
LIB_INC_PRV := $(LWE_DIR)/inc/private
LCF := $(LWE_DIR)/lwe.lds

LWE_HEAD := $(wildcard $(LWE_DIR)/src/init/*.S)
LWE_START := $(wildcard $(LWE_DIR)/src/init/*.c)
LWE_COMMON := $(wildcard $(LWE_DIR)/src/common/*.c)
LWE_SWI := $(wildcard $(LWE_DIR)/src/swi/*.c)
LWE_TIMER_WHEEL := $(wildcard $(LWE_DIR)/src/timer_wheel/*.c)
LWE_HWI := $(wildcard $(LWE_DIR)/src/hwi/*.c) 
LWE_CACHE := $(wildcard $(LWE_DIR)/src/cache/*.c)
LWE_QMAN := $(wildcard $(LWE_DIR)/src/qman/*.c)
LWE_BMAN := $(wildcard $(LWE_DIR)/src/bman/*.c)
LWE_CRC := $(wildcard $(LWE_DIR)/src/crc/*.c)
LWE_CPU_USAGE := $(wildcard $(LWE_DIR)/src/cpu_usage/*.c)
LWE_SEC40 := $(wildcard $(LWE_DIR)/src/sec40/*.c)
LWE_WDOG := $(wildcard $(LWE_DIR)/src/wdog/*.c)
LWE_PME := $(wildcard $(LWE_DIR)/src/pme2/*.c)
LWE_HELLO_TEST := test/hello/hello.c
LWE_HELLO_ALT_TEST := test/hello_alt/hello.c
LWE_QMAN_TEST := test/qman/qman.c
LWE_BMAN_TEST := test/bman/bman.c
LWE_PME2_TEST := test/pme2/pme2.c
LWE_SCAN_TEST := test/scan/scan.c
LWE_SPEED_TEST := test/speed/speed.c
LWE_BLASTMAN_TEST := test/blastman/blastman.c
LWE_ALLOC_TEST := test/alloc/alloc.c
LWE_CGR_TEST := test/cgr/cgr.c
LWE_VDQCR_TEST := test/vdqcr/vdqcr.c

CC := $(CROSS_COMPILE)$(CC)
AS := $(CROSS_COMPILE)$(AS)

CC_OPTS= $(CC_CPU_FLAGS) $(AS_CPU_FLAGS) -m32 -nostdinc -I. \
        -I$(LIB_INC_PRV) -I$(LIB_INC_PUB) -I$(LIBFDT_INC) \
		-I$(HV_INC) -std=gnu99 -include $(HV_DEP_H)

WARNING_OPTS_C = -Wall \
  -Wextra \
  -Wstrict-prototypes \
  -Waggregate-return \
  -Wdisabled-optimization \
  -Wunused \
  -Winit-self \
  -Wno-unknown-pragmas

# The warning options given below would be turned ON later. Right now code does not compile with them
#  -Werror
#  -Wshadow
#  -Wundef
#  -Wswitch-default
#  -Wmissing-declarations
#  -Wconversion
#  -Wredundant-decls
#  -Winline

CC_OPTS_C=$(BUILD_FLAGS) $(WARNING_OPTS_C) \
  -c \
  -g \
  -fno-strict-aliasing \
  -fno-common \
  -pipe \
  -ffixed-r2 \
  -fno-stack-protector \
  -mno-string \
  -mmultiple \
  -mno-altivec

PPC_INC= -I$(LIB_INC) -I$(LIB_INC_PUB)

CONFIG_SWI=y
CONFIG_HWI=y
CONFIG_TIMER_WHEEL=y
CONFIG_CACHE_LOCK=y
CONFIG_QMAN=y
CONFIG_BMAN=y
CONFIG_CRC=y
CONFIG_CPU_USAGE=y
CONFIG_SEC40=y
CONFIG_WDOG=y
CONFIG_PME=y

CC_OPTS_ASM=-D_ASM -Ibin/src -include $(HV_BIN)/assym.s 
LD := $(CROSS_COMPILE)$(LD)
LD_OPTS=-Wl,-EB -Wl,-Bstatic $(CC_CPU_FLAGS) -nostdlib -Wl,-Map -Wl,bin/debug.map -Xlinker -wrap -Xlinker memcpy
GENASSYM=$(HV_DIR)/genassym.sh
MKDIR=mkdir -p

HV_SRCS := fsl-booke-tlb.c exceptions.S trap.c malloc.c sprintf.c

lwe-src-$(CONFIG_SWI) += $(LWE_SWI) 
lwe-src-$(CONFIG_HWI) += $(LWE_HWI)
lwe-src-$(CONFIG_TIMER_WHEEL) += $(LWE_TIMER_WHEEL)
lwe-src-$(CONFIG_CACHE_LOCK) += $(LWE_CACHE)
lwe-src-$(CONFIG_QMAN) += $(LWE_QMAN)
lwe-src-$(CONFIG_BMAN) += $(LWE_BMAN)
lwe-src-$(CONFIG_CRC) += $(LWE_CRC)
lwe-src-$(CONFIG_CPU_USAGE) += $(LWE_CPU_USAGE)
lwe-src-$(CONFIG_SEC40) += $(LWE_SEC40)
lwe-src-$(CONFIG_WDOG) += $(LWE_WDOG)
lwe-src-$(CONFIG_PME) += $(LWE_PME)

TEST := $(subst lwe/,,$(TEST))

SRCS := $(LWE_HEAD:%=%) $(HV_SRCS:%=hv/%) $(LWE_START:%=%) \
		$(LWE_COMMON:%=%) $(lwe-src-y:%=%) 
OBJS := $(basename $(SRCS))
OBJS := $(OBJS:%=%.o)
OBJS := $(OBJS:%=bin/%)
OBJS := $(subst ../../,,$(OBJS))
OBJS := $(subst ../,,$(OBJS))

TEST_SRCS := $(TEST:%=lwe/%)
TEST_OBJS := $(basename $(TEST_SRCS))
TEST_OBJS := $(TEST_OBJS:%=%.o)
TEST_OBJS := $(TEST_OBJS:%=bin/%)
TEST_OBJS := $(subst ../../,,$(TEST_OBJS))
TEST_OBJS := $(subst ../,,$(TEST_OBJS))

####################

# compile and gen dependency file

$(LWE_BIN)/%.d : $(LWE_DIR)/%.c
	$(MKDIR) `dirname $@`
	OBJ=`echo $@ | sed 's/\.d$$/\.o/'` && \
	$(CC) -MM -MP $(LIBC_INC) $(LIBC_CC_OPTS) $(PPC_INC) $(CC_OPTS) $(CC_OPTS_C) -MF "$@" -MT "$$OBJ" $<

$(LWE_BIN)/%.d : $(LWE_DIR)/%.S
	$(MKDIR) `dirname $@`
	OBJ=`echo $@ | sed 's/\.d$$/\.o/'` && \
	$(CC) -MM -MP $(PPC_INC) $(CC_OPTS) $(CC_OPTS_ASM) $(CC_OPTS_C) -MF "$@" -MT "$$OBJ" $<

$(LWE_BIN)/%.o : $(LWE_DIR)/%.c $(LWE_BIN)/%.d
	$(CC) $(PPC_INC) $(LIBC_INC) $(LIBC_CC_OPTS) $(CC_OPTS) $(CC_OPTS_C) -c -o $@ $< 

$(LWE_BIN)/%.o : $(LWE_DIR)/%.S $(LWE_BIN)/%.d
	@$(MKDIR) `dirname $@`
	$(CC) $(PPC_INC) $(CC_OPTS) $(CC_OPTS_ASM) $(CC_OPTS_C) -c -o $@ $< 

$(HV_BIN)/%.d : $(HV_DIR)/%.c
	@$(MKDIR) $(@D)
	OBJ=`echo $@ | sed 's/\.d$$/\.o/'` && \
	$(CC) -M $(LIBC_INC) $(LIBC_CC_OPTS) $(CC_OPTS) $(CC_OPTS_C) -MF "$@" -MT "$$OBJ" $< 

$(HV_BIN)/%.o : $(HV_DIR)/%.c $(HV_BIN)/%.d
	$(CC) $(LIBC_INC) $(LIBC_CC_OPTS) $(CC_OPTS) $(CC_OPTS_C)  -c -o $@ $<

$(HV_BIN)/%.d : $(HV_DIR)/%.S $(HV_BIN)/assym.s
	@$(MKDIR) $(@D)
	OBJ=`echo $@ | sed 's/\.d$$/\.o/'` && \
	$(CC) -M $(CC_OPTS) $(CC_OPTS_ASM) -MF "$@" -MT "$$OBJ" $< 

$(HV_BIN)/%.o : $(HV_DIR)/%.S $(HV_BIN)/%.d
	$(CC) $(CC_OPTS) $(CC_OPTS_ASM) -c -o $@ $<

$(HV_BIN)/assym.d: $(HV_DIR)/assym.c
	@$(MKDIR) $(@D)
	OBJ=`echo $@ | sed 's/\.d$$/\.o/'` && \
	$(CC) -M $(LIBC_INC) $(LIBC_CC_OPTS) $(CC_OPTS) $(CC_OPTS_C) -MF "$@" \
	-MT "$$OBJ" $< 

$(HV_BIN)/assym.o: $(HV_DIR)/assym.c $(HV_BIN)/assym.d
	$(CC) $(LIBC_INC) $(LIBC_CC_OPTS) $(CC_OPTS) -c -o $@ $<

$(HV_BIN)/assym.s: $(HV_BIN)/assym.o
	@$(MKDIR) $(@D)
	$(GENASSYM) -o $@ $<
