/*
 * Copyright (c) 2009, 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __CONSOLE_H__
#define __CONSOLE_H__

#include <stdint.h>
#include <stdbool.h>
#include <ns16550.h>

#define INV_CONSOLE_ID	255
#define MAX_NUM_CONSOLES	8

extern ns16550 uart_obj[NUM_OF_UARTS];

enum console_drv_t {
	INVALID,
	HV_BYTE_CHANNEL,
	UART
};

typedef void (*stdin_intr_handler_t)(void);

struct console_t {
	bool console_ok;
	enum console_drv_t driver;
	uint32_t log_lock;
	int ch_node;
	uint32_t ch_handle;
	uint32_t stdin_vcpu;
	stdin_intr_handler_t stdin_handler;
};

extern struct console_t lwe_console[MAX_NUM_CONSOLES];


int init_consoles(void);
int subscribe_console(void);
int console_log(const char *str, size_t log_size);
int attach_console_input(stdin_intr_handler_t stdin_handler);
int console_put(uint8_t *str, size_t size);
int console_gets(char *cmd_buf, uint32_t len);


#endif /*__CONSOLE_H__*/

