/** @file
 * Exception handling registration support
 */

/*
 * Copyright (c) 2009 - 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __EXCEPTION_PRIVATE_H
#define __EXCEPTION_PRIVATE_H

#include <stdint.h>
#include <exception.h>
#include <nmi.h>

/* Holds exception handler function and it's argument
 * registered by user, otherwise a default handler
 */
struct exc_intr {
	exc_func handler;
	void *arg;
};

struct pamu_error {
	uint32_t lpid;
	uint64_t access_violation_addr;
	uint32_t avs1;
	uint32_t avs2;
	uint32_t liodn_handle;
};

struct hv_error {
	char domain[32];
	char error[128];
	char hdev_tree_path[256];
	char gdev_tree_path[256];
	struct pamu_error pamu;
};

extern nmi_hndlr reg_nmi_hndlr;
void exception_handler_init(void);
void lwe_exception_handler(struct trapframe *regs);

#endif				/* __EXCEPTION_PRIVATE_H */
