/** @file
 * Hardware Interrupts setup and control.
 */

/*
 * Copyright (c) 2008 - 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __HWI_PRIVATE_H
#define __HWI_PRIVATE_H

#include <stdbool.h>
#include <stdint.h>
#include <hwi.h>

#define INT_CONFIG	1
#define INT_PRIORITY	15
#define INT_DISABLE	1
#define INT_ENABLE	0
/* Hypervisor supports maximum of 1024 interrupts currently.
   Picked up from MAX_HANDLES macro in Hypervisor */
#define MAX_INTERRUPTS	1024
#define MPIC_EOI	0x00B0
#define MPIC_DIRECT_MASK	0x0004

struct hwi_intr {
	hwi_func handler;
	void *arg;
};

struct hwi_global_s {
	/*coreint mode or legacy mode */
	uint32_t legacy;
	/* Set if direct_eoi window opened by hypervisor */
	uint32_t direct_eoi;
	uint32_t lock_ptr;
	/* Set to true if direct eoi is poosible for a interrupt */
	bool hwi_eoi[MAX_INTERRUPTS];
};

struct hwi_percpu_s {
	/* Used to store interrupt handler for each interrupt */
	struct hwi_intr lwe_hwi[MAX_INTERRUPTS];
	/* Virtual address of mpic window if opened by hypervisor */
	void *mpic_va;
	void *mpic_eoi;
};

int hwi_init(int core);

#endif
