/** 
\file
\brief	This file provides cache locking and unlocking support for L1 and L2
	cache.  L3 cache management is not supported as of now.
*/

/*
 * Copyright (c) 2008-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __CACHE_H__
#define __CACHE_H__

#ifdef __cplusplus
    extern "C" {
#endif

#include <stdint.h>

#define CACHE_LINE_SIZE	64

enum cache_type_t {
	L1_DATA_CACHE,
	L1_INST_CACHE,
	L2_CACHE
};

enum cache_target_t {
	CT_L1_CACHE,
	CT_L2_CACHE = 2
};

/**
\brief		Lock L1, L2 data cache line.

\param[in]	addr_start - Start address of the data cache to be locked

\param[in]	size - Size of the data cache to be locked

\param[in]	ct - CT_L1_CACHE for L1-data cache,
				 CT_L2_CACHE for L2 cache

\return   Success: 0; Fail: Non-zero
*/
int dcache_lock(void *addr_start, uint32_t size, enum cache_target_t);

/** 
\brief		Unlock L1, L2 data cache line.

\param[in]	addr_start - Start address of the data cache to be locked

\param[in]	size - Size of the data cache to be locked

\param[in]	ct - CT_L1_CACHE for L1-data cache,
				 CT_L2_CACHE for L2 cache

\return   Success: 0; Fail: Non-zero
*/
int dcache_unlock(void *addr_start, uint32_t size, enum cache_target_t);

#ifdef __cplusplus
    }
#endif
#endif /*__CACHE_H__*/
