/** @file
 * Build configuration.
 */

/*
 * Copyright (c) 2008-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __CONF_H
#define __CONF_H

#ifdef __cplusplus
    extern "C" {
#endif

/***********/
/* General */
/***********/

#ifndef LWE_LOG_LEVEL	/**< Setting default Log Level */

#ifdef DEBUG
#define LWE_LOG_LEVEL	LWE_LOG_LEVEL_DEBUG
#else
#define LWE_LOG_LEVEL 	LWE_LOG_LEVEL_INFO
#endif

#endif

/* support for BUG_ON()s, might_sleep()s, etc */
#undef CONFIG_BUGON

/* try to do better than memcpy() */
#define CONFIG_TRY_BETTER_MEMCPY

/* don't support blocking (so, WAIT flags won't be #define'd) */
#undef CONFIG_FSL_DPA_CAN_WAIT

#ifdef CONFIG_FSL_DPA_CAN_WAIT
/* if we can "WAIT" - can we "WAIT_SYNC" too? */
#undef CONFIG_FSL_DPA_CAN_WAIT_SYNC
#endif

/* don't do any portal logic in interrupts (compiles out irq_save/restore) */
#undef CONFIG_FSL_DPA_IRQ_SAFETY

/********/
/* Bman */
/********/

/* support for run-time parameter checking, assertions, etc */
#undef CONFIG_FSL_BMAN_CHECKING

#ifdef CONFIG_FSL_DPA_IRQ_SAFETY
/* support IRQs, whether or not they're used at run-time */
#define CONFIG_FSL_BMAN_HAVE_IRQ
#endif

#ifdef CONFIG_FSL_BMAN_HAVE_IRQ
/* do slow-path processing via IRQ */
#define CONFIG_FSL_BMAN_PIRQ_SLOW
#endif

/********/
/* Qman */
/********/

/* workarounds for errata and missing features in p4080 rev1 */
#define CONFIG_FSL_QMAN_BUG_AND_FEATURE_REV1

/* don't use rev1-specific adaptive "backoff" for EQCR:CI updates */
#undef CONFIG_FSL_QMAN_ADAPTIVE_EQCR_THROTTLE

/* support for run-time parameter checking, assertions, etc */
#undef CONFIG_FSL_QMAN_CHECKING

/* support FQ allocator built on top of BPID 0 */
#define CONFIG_FSL_QMAN_FQALLOCATOR

/* don't disable DCA on auto-initialised portals */
#undef CONFIG_FSL_QMAN_PORTAL_DISABLEAUTO_DCA

/* Interrupt-gating settings */
#define CONFIG_FSL_QMAN_PIRQ_DQRR_ITHRESH 0
#define CONFIG_FSL_QMAN_PIRQ_MR_ITHRESH 0
#define CONFIG_FSL_QMAN_PIRQ_IPERIOD 0

/* maximum number of DQRR entries to process in qman_poll() */
#define CONFIG_FSL_QMAN_POLL_LIMIT 8

#ifdef CONFIG_FSL_DPA_IRQ_SAFETY
/* support IRQs, whether or not they're used at run-time */
#define CONFIG_FSL_QMAN_HAVE_IRQ
#endif

#ifdef CONFIG_FSL_QMAN_HAVE_IRQ
/* do slow-path processing via IRQ */
#define CONFIG_FSL_QMAN_PIRQ_SLOW
/* do not do fast-path processing via IRQ */
#undef CONFIG_FSL_QMAN_PIRQ_FAST
#endif

/* don't compile support for NULL FQ handling */
#undef CONFIG_FSL_QMAN_NULL_FQ_DEMUX

/* don't compile support for DQRR prefetching (so stashing is required) */
#undef CONFIG_FSL_QMAN_DQRR_PREFETCHING


/*Timer wheel configuration*/
#define TICK_SIZE_MS	10	/**< Timer wheel tick size in millisecond*/

#define RECLAIM_QUEUE_LEN 256	/**< Length of timer wheel reclaimation queue*/

#define NUM_TIMER_OBJECTS	1024	/*Number of timers supported per core*/

#define MAX_TIMEOUT_MS		0x80000000 /*Max possible timeout*/


#define INTEG_DURATION_MS	500ULL /* CPU usage
					Integration interval in ms */

#ifdef __cplusplus
    }
#endif
#endif
