/**
\file
\brief	Execption handler registration support.
	This file provides a hook to user to register handlers 
	for the exceptions provided in e500mc model except for
	machine check, external interrupt, program interrupt,
	decrementer and watchdog .
	For rest of the exections if teh handlre is not registerd and 
	that exception ocurrs during the program flow, default handler 
	is invoked and the code excetion enters a while(1) loop.
	
	printf statement should not be used from exception handler code.
	
*/

/*
 * Copyright (c) 2009, 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __EXCEPTION_H
#define __EXCEPTION_H

#ifdef __cplusplus
    extern "C" {
#endif
#include <stdint.h>
#include <io.h>
#include <trapframe.h>

typedef int (*exc_func) (struct trapframe *, void *);

/**
\brief		Exception handler registration. 
 		Exception handler for following IVORs are used by LWE
		and thereby excluded from registration -
		machine check, external interrupt, program interrupt,
		decrementer and watchdog 

\param[in]	vector - Interrupt vector

\param[in] 	handler - Handler function to be attached with interrupt vector

\param[in]  	arg - Argument of handler function

\return   	0 on success, otherwise -ve value

\note		Use of printf statement form theregistered handler code is not
		supported
*/
int register_exception_handler(uint16_t vector, exc_func handler, void *arg);

#ifdef __cplusplus
    }
#endif
#endif				/* __EXCEPTION_H */
