/** @file
 * Prototypes of lwe initialization libraries
 */

/*
 * Copyright (c) 2008-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __INIT_H__
#define __INIT_H__

#ifdef __cplusplus
    extern "C" {
#endif
#include <stdio.h>
#include <stdint.h>
#include "types.h"

#define __PERCPU __attribute__ ((section(".percpu")))
#define __STATS __attribute__ ((section(".stats_data")))

#define MAX_CORES	8

extern int main(int argc, char **argv);

/** devtree_ptr - Guest device tree pointer as passed from the hypervisor
 */
extern void *devtree_ptr;
extern uint32_t guest_cache_lock_on;
extern uint8_t g_num_cores;
extern uint32_t sec_core_mask;
extern uint32_t online_core_mask;

extern __PERCPU volatile int in_irq;
extern __PERCPU uint32_t lwe_instance;
extern __PERCPU uint32_t l1_data_cache_blk_size;
extern __PERCPU uint32_t l1_inst_cache_blk_size;
extern __PERCPU uint32_t l2_cache_blk_size;
extern __PERCPU uint32_t timebase_freq;

/**
\brief Returns the free tlb1 index on the core running this method

\return
	0x0-0xf	: Valid Index
	0xff	: Error Value
*/
extern uint32_t get_freetlb1_idx(void);

typedef int (*entryp) (int argc, char *argv[]);

/**
\brief   Boot core can call this function to start a function on idle cpu

\param[in] vcpu_id : VCPU to start
\param[in] func : VCPU Entry point
\param[in] argc : Number of Arguments to be passed to entry point
		  Maximum number of argumnets allowed is 255
\param[in] argv : List of arguments to be passed to entry point
		  If the string length of each argv exceeds 255 bytes, only
		  the first 255 bytes will be passed to the func
\return
	0x0	: VCPU started successfully
	-1	: Error in starting vcpu
*/
int start_cpu(uint8_t vcpu_id, entryp func, int argc, char *argv[]);


#ifdef __cplusplus
    }
#endif
#endif /*__INIT_H__*/
