/**
\file
\brief	intra-partition signaling support
	This file provides API for communication between cores
	in a partition.

	register_ip_handler() can be used to register handler on core which
	would be invoked when some other core signals this core.        
	One core can signal another core using send_ip_sgnl().

*/

/*
 * Copyright (c) 2009, 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __INTRA_SGNL_H
#define __INTRA_SGNL_H

#ifdef __cplusplus
    extern "C" {
#endif
#include <stdint.h>
#include <trapframe.h>
#include <trap_booke.h>

typedef int (*ip_func) (struct trapframe *, void *);
/**
\brief		register_ip_handler can be used to register handler on core
		which would be invoked when some other core signals this core.

\param[in]	handler - Handler that will be invoked when intra-partition
		signal from another core is received

\param[in]	arg - Argument of handler function  	

\return		0 for success, -ve value for failure
\note		Use of printf statement form the registered handler code is not
		supported
*/
int register_ip_handler(ip_func handler, void *arg);

/**
\brief		One core can trigger another core or cores within the partition

\param[in]	core_mask - Mask of core(s)

\return		Void
*/
int send_ip_sgnl(uint8_t core_mask);

#ifdef __cplusplus
    }
#endif
#endif				/* __INTRA_SGNL_H */
