/** @file
 * This file provides some miscellaneous APIs
 */

/*
 * Copyright (c) 2008 - 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __MISC_H__
#define __MISC_H__

#ifdef __cplusplus
    extern "C" {
#endif
#include <stdint.h>
#include "fsl-booke-tlb.h"
#include <stdbool.h>
#include "init.h"

#define BITS_PER_BYTE (8)
#define BYTES_PER_WORD (4)

extern uint32_t lwe_instance;
extern uint32_t online_core_mask;

/**
\brief		This function provides the LWE partition's name

\param[in] 	ecode - error code pointer, populated with error code
		in case of failure

\return		Returns the name/label of the partition
		from which this function is called.
		NULL- In case of failure
*/
const char *get_part_name(int *ecode);

int get_fdt_mem_ranges(int node, phys_addr_t *start, phys_addr_t *size);

/** 
\brief		This function provided the LWE core id	

\return		Returns the id of the core from which this function is called.	
*/
static inline uint32_t
get_lwe_id(void)
{
	return lwe_instance;
}

/** 
\brief		This function provided the mask of cores which are in main 
		function

\return		Returns the mask specifiying which cores are up and are in
		the main function in partition. Once a particular core exits
		from the main function, it would no longer be viible in this mask.

\note		mask [0 - 31 bits] - LSB(Bit 31), MSB (Bit 0) 
		Bit 31 -- VCPU 0 Bit 30 -- VCPU 1 and so on
*/
static inline uint32_t
get_online_core_mask(void)
{
	return online_core_mask;
}

/**
\brief		This function provided the number of cores in the partition

\return		Returns the number of cores in the partition
*/
static inline uint8_t get_num_cores(void)
{
	return g_num_cores;
}

#ifdef __cplusplus
    }
#endif
#endif /*__MISC_H__*/
