/** @file
 * Contains Performance Monitoring APIs.
 * This file provides API's to monitor the performance of different types of
 * event.
 *
 */
/*
 * Copyright (c) 2009, 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __PERF_MON_H__
#define __PERF_MON_H__

#ifdef __cplusplus
    extern "C" {
#endif
enum PM_event {
	PM_NOTHING = 0,
	PM_CYCLE,
	PM_INSTRUCTIONS_COMPLETED,
	PM_MICRO_OPS_COMPLETED,
	PM_INSTRUCTIONS_FETCHED,
	PM_MICRO_OPS_DECODED,
	PM_EVENTS_TRANSITIONS,
	PM_EVENTS_CYCLES,
	PM_BRANCH_INSTRUCTIONS_COMPLETED,
	PM_LOAD_MIRCRO_OPS_COMPLETED,
	PM_STORE_MIRCRO_OPS_COMPLETED,
	PM_NUMBER_OF_CQ_REDIRECTS,
	PM_BRANCHES_FINISHED,
	PM_TAKEN_BRANCHES_FINISHED,
	PM_FINISHED_UNCONDITIONAL_BRANCHES,
	PM_BRANCHES_MISPREDICTED,
	PM_BTB_BRANCHES_MISPREDICTED,
	PM_BTB_HITS_AND_PSEUDO_HITS,
	PM_CYCLES_DECODE_STALLED,
	PM_CYCLES_ISSUE_STALLED,
	PM_CYCLES_BRANCH_ISSUE_STALLED,
	PM_CYCLES_SU1_SCHEDULE_STALLED,
	PM_CYCLES_SU2_SCHEDULE_STALLED,
	PM_CYCLES_MU_SCHEDULE_STALLED,
	PM_CYCLES_LRU_SCHEDULE_STALLED,
	PM_CYCLES_BU_SCHEDULE_STALLED,
	PM_TOTAL_TRANSLATED,
	PM_LOADS_TRANSLATED,
	PM_STORES_TRANSLATED,
	PM_TOUCHES_TRANSLATED,
	PM_CACHEOPS_TRANSLATED,
	PM_CACHE_INHIBITED_ACCESSES_TRANSLATED,
	PM_GUARDED_LOADS_TRANSLATED,
	PM_WRITE_THROUGH_STORES_TRANSLATED,
	PM_MISALIGNED_LOAD_OR_STORES_ACCESSES_TRANSLATED,
	PM_TOTAL_ALLOCATED_TO_DLFB,
	PM_LOADS_TRANSLATED_AND_ALOCATED_TO_DLFB,
	PM_STORES_COMPLETED_AND_ALOCATED_TO_DLFB,
	PM_TOUCHES_TRANSLATED_AND_ALOCATED_TO_DLFB,
	PM_STORES_COMPLETED,
	PM_DATA_L1_CACHE_LOCKS,
	PM_DATA_L1_CACHE_RELOADS,
	PM_DATA_L1_CACHE_CASTOUTS,
	PM_LOAD_MISS_WITH_DLFB_FULL,
	PM_LOAD_MISS_WITH_LOAD_QUEUE_FULL,
	PM_LOAD_GUARDED_MISS,
	PM_TRANSLATE_STORE_WHEN_STORE_QUEUE_FULL,
	PM_ADRESS_COLLISION,
	PM_DATA_MMU_MISS,
	PM_DATA_MMU_BUSY,
	PM_MISALIGNED_ACCESS_WHEN_FIRST_PART_MISSED_IN_CACHE,
	PM_LOAD_MISS_WITH_DLFB_FULL_51,
	PM_LOAD_MISS_WITH_LOAD_QUEUE_FULL_52,
	PM_LOAD_GUARDED_MISS_53,
	PM_TRANSLATE_STORE_WHEN_STORE_QUEUE_FULL_54,
	PM_ADRESS_COLLISION_55,
	PM_DATA_MMU_MISS_56,
	PM_DATA_MMU_BUSY_57,
	PM_MISALIGNED_ACCESS_WHEN_FIRST_PART_MISSED_IN_CACHE_58,
	PM_INSTRUCTION_L1_CACHE_LOCKS,
	PM_INSTRUCTION_L1_CACHE_RELOADS_FROM_FETCH,
	PM_NUMBER_OF_FETCHS,
	PM_INSTRUCTION_MMU_TLB4K_RELOADS,
	PM_INSTRUCTION_MMU_VSP_RELOADS,
	PM_DATA_MMU_TLB4K_RELOADS,
	PM_DATA_MMU_VSP_RELOADS,
	PM_L2MMU_MISSES,
	PM_BIU_MASTER_REQUESTS,
	PM_BIU_MASTER_INSTRUCTION_SIDE_REQUESTS,
	PM_BIU_MASTER_DATA_SIDE_REQUESTS,
	PM_BIU_MASTER_DATA_SIDE_CASTOUT_REQUESTS,
	PM_BIU_MASTER_RETRIES,
	PM_SNOOP_REQUESTS,
	PM_SNOOP_HITS,
	PM_SNOOP_PUSHES,
	PM_SNOOP_RETRIES,
	PM_DLFB_LOAD_MISS_CYCLES,
	PM_ILFB_FETCH_MISS_CYCLES,
	PM_EXTERNAL_INPUT_INTERRUPT_LATENCY_CYCLES,
	PM_CRITICAL_INPUT_INTERRUPT_LATENCY_CYCLES,
	PM_EXTERNAL_INPUT_INTERRUPT_PENDING_LATENCY_CYCLES,
	PM_CRITICAL_INPUT_INTERRUPT_PENDING_LATENCY_CYCLES,
	PMC0_OVERFLOW,
	PMC1_OVERFLOW,
	PMC2_OVERFLOW,
	PMC3_OVERFLOW,
	PM_INTERRUPT_TAKEN,
	PM_EXTRENAL_INPUT_INTERRUPTS_TAKEN,
	PM_CRITICAL_INPUT_INTERRUPTS_TAKEN,
	PM_SYSTEM_CALL_AND_TRAP_INTERRUPTS,
	PM_TBL_TRANS,
	PM_L2_LINEFILL,
	PM_L2_VS,
	PM_CASTOUT_RELEASED,
	PM_INTV_ALLOCATIONS,
	PM_STORE_RETRIES_TO_MBAR,
	PM_STORE_RETRIES_DUE_TO_MISC,
	PM_STASH_L1_HIT,
	PM_STASH_L2_HIT,
	PM_STASH_BUSY_1,
	PM_STASH_BUSY_2,
	PM_STASH_BUSY_3,
	PM_STASH_HIT,
	PM_STASH_HIT_DLFB,
	PM_STASH_DOWNGRADE_1,
	PM_STASH_DOWNGRADE_2,
	PM_STASH_REQUESTS,
	PM_STASHES_TO_L1_DATA_CACHE,
	PM_STASHES_TO_BACKSIDE_L2,
	PM_STALLS_DUE_TO_NO_CAQ_OR_COB,
	PM_L2_CACHE_ACCESSES,
	PM_L2_HIT_CACHE_ACCESSES,
	PM_L2_CACHE_DATA_ACCESSES,
	PM_L2_CACHE_DATA_HITS,
	PM_L2_CACHE_INSTR_ACCESSES,
	PM_L2_CACHE_INSTR_HITS,
	PM_L2_CACHE_ALLOCATIONS,
	PM_L2_CACHE_DATA_ALLOCATIONS,
	PM_L2_CACHE_DIRTY_DATA_ALLOCATIONS,
	PM_L2_CACHE_INSTR_ALLOCATIONS,
	PM_L2_CACHE_UPDATES,
	PM_L2_CACHE_CLEAN_UPDATES,
	PM_L2_CACHE_DIRTY_UPDATES,
	PM_L2_CACHE_CLEAN_REDUNDANT_UPDATES,
	PM_L2_CACHE_DIRTY_REDUNDANT_UPDATES,
	PM_L2_CACHE_LOCKS,
	PM_L2_CACHE_CASTOUTS,
	PM_L2_CACHE_DATA_DIRTY_HITS,
	PM_INSTR_LFBWENT_HIGH_PRIORITY,
	PM_SNOOP_THROTTLING_TURNED_ON,
	PM_L2_INVALIDATION_OF_CLEAN_LINES,
	PM_L2_INVALIDATION_OF_INCOHERENT_LINES,
	PM_L2_INVALIDATION_OF_COHERENT_LINES,
	PM_COHERENT_LOOKUP_MISS,
	PM_IAC1_DETECTED = 140,
	PM_IAC2_DETECTED,
	PM_DAC1_DETECTED = 144,
	PM_DAC2_DETECTED,
	PM_DVT0_DETECTED = 148,
	PM_DVT1_DETECTED,
	PM_DVT2_DETECTED,
	PM_DVT3_DETECTED,
	PM_DVT4_DETECTED,
	PM_DVT5_DETECTED,
	PM_DVT6_DETECTED,
	PM_DVT7_DETECTED,
	PM_CYCLES_COMPLETION_STALLED,
	PM_FPU_DOUBLEPUMP = 160,
	PM_FPU_FINISH,
	PM_FPU_DIVIDE_CYCLES,
	PM_FPU_DENORM_INPUT,
	PM_FPU_RESULT_STALL,
	PM_FPU_FPSCR_FULL_STALL,
	PM_FPU_PIPE_SYNC_STALL,
	PM_FPU_INPUT_DATA_STALL,
	PM_DECORATED_LOADS = 176,
	PM_DECORATED_STORES,
	PM_LOAD_RETRIES,
	PM_STWCX_SUCCESSES,
	PM_STWCX_UNSUCCESSFUL
};

typedef uint32_t pmr_handle_t;

enum PME_state {
	PM_ENABLE,
	PM_DISABLE
};

enum PMI_state {
	PMI_DISABLE,
	PMI_ENABLE
};


struct pmes {
	uint8_t event;
	enum PME_state state;
	uint32_t threshold;
	uint32_t threshmul;
	enum PMI_state pmi;
};

/**
\brief		Intialize various performance monitoring registers/counters
*/
void initPM(void);

/**
\brief		Configure an event for monitoring the performance

\param[in]	event - event that needs to be configured.

\param[in]	initial_state - initial state of monitoring i.e, start/stop

\param[in]	threshold - threshold value (0 - 63)

\param[in]	threshmul - threshold value multiplier (2^n where n 0 t0 7)

\param[in]	pmi - initial state of interrupt i.e, enabled/disabled

\param[out]	handle - handle to be used for event

\return		on sucess it returns 0,
		on failure, it returns error code.
*/
int configurePM_event(enum PM_event event, enum PME_state initial_state,
		      uint32_t threshold, uint32_t threshmul,
		      enum PMI_state pmi, pmr_handle_t *handle);

/**
\brief		Unconfigure performance monitoring of an event.

\param[in]	handle - handle to an earlier configured event that needs to
		be unconfigured.

\return		on sucess it returns 0,
		on failure, it returns error code.
*/
int unconfigurePM_event(pmr_handle_t handle);

/**
\brief		Start monitoring of an earlier configured event.

\param[in]	handle - handle of event for which monitoring needs to be
		started

\return		On success, it returns 0,
		On failure, it returns error code.
*/
int startPM_event(pmr_handle_t handle);

/**
\brief		Stop monitoring of an earlier configured event.

\param[in]	handle - handle of event for which monitoring needs to be
		stopped

\return		on sucess it returns 0,
		on failure, it returns error code.
*/
int stopPM_event(pmr_handle_t handle);

/**
\brief		Re-starts(starts) the monitoring of event after setting the
		value of performance counter to zero

\param[in]      handle - handle of event for which monitoring needs to be
		re-started.
\return         On success, it returns 0,
		On failure, it returns error code.
*/
int restartPM_event(pmr_handle_t handle);

/**
\brief		Get the current value of performance counter for the specified
		event.

\param[in]	handle - handle of event for whome counter value needs to be
		returned

\param[out]	val - performance counter value.

\return         On success, it returns 0,
		On failure, it returns error code.
*/
int getPM_event_count(pmr_handle_t handle, uint32_t *val);

/**
\brief		Get all configured events.

\param[out]	event_array - array populated with presently configured events,
		Value 0, of an array element, represents no event configured
		for monitoring. At most four events can be configured for
		monitoring at any point of time.

\return         On success, it returns 0,
		On failure, it returns error code.
*/
int getPM_configured_events(uint32_t *event_array);

/**
/brief		Set the threshold value of an earlier configured event.

\param[in]      threshold - threshold value (0 - 63)

\param[in]      threshmul - threshold value multiplier (2^n where n 0 t0 7)

\return		On success, it returns 0,
		On failure, it returns error code.
*/
int setPM_event_threshold(pmr_handle_t handle, uint32_t threshold,
			  uint32_t threshmul);
/**
/brief		Get status of an earlier configured event.

\param[in]      handle - handle of event for whom status needs to be returned

\param[out]	status - status of specified monitored event. Status will
		constitue of threshold value, state(monitoring enable/disabled),
		interrupt enable/disabled.

\return         On success, it returns 0,
		On failure, it returns error code.
*/
int getPM_event_status(pmr_handle_t handle, struct pmes *status);

/**
\brief		Enable the performance monitoring interrupt.

\param[in]      handle - handle of event for which performance monitoring
		interrupt needs to be enabled.

\return		On success, it returns 0,
		On failure, it returns error code.
*/
int enablePM_event_interrupt(pmr_handle_t handle);

/**
\brief          Disable the performance monitoring interrupt.

\param[in]      handle - handle of event for which performance monitoring
		interrupt needs to be disabled.

\return         On success, it returns 0,
		On failure, it returns error code.
*/
int disablePM_event_interrupt(pmr_handle_t handle);

#ifdef __cplusplus
    }
#endif
#endif /* __PERF_MON_H__ */
