/** @file
 * Stack related functions. 
 */

/*
 * Copyright (c) 2008-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __STACK_EXT_H
#define __STACK_EXT_H

#ifdef __cplusplus
    extern "C" {
#endif
#include <stdbool.h>

struct stack_t;

/**
 * \brief Initializes the stack by allocating memory for max number of entries.
 * \param[in] max_extries : max entries to be allocated on stack.
 * \param[in] p_ecode : error code pointer. 
 * \return Returns the stack handle on success, NULL if failed. 
 */
struct stack_t *stack_init(uint32_t max_entries, uint32_t * p_ecode);

/** Pushes an object into the stack. 
 * \param[in] is_private : Tells whether the stack is private accessed on 
	caller's core or being accessible through all cores.
 * \param[in] p_stack : pointer to the stack
 * \param[in] p_item : pointer to the item to be pushed on stack
 * \return 0 for success, -ve error code value for failure
*/
int stack_push(struct stack_t *p_stack, void *p_item);

/**
 * \brief Pops an object from the stack. 
 * \param[in] p_stack : pointer to the stack
 * \return void * : pointer to the item popped
 */
void *stack_pop(struct stack_t *p_stack);

/**
 * \brief Frees the stack object.
 * \param[in] p_stack : pointer to the stack
 * \note Assumes all nodes correctly returned to stack before this 
 *	function is called
 */
int stack_teardown(struct stack_t *p_stack);

#ifdef __cplusplus
    }
#endif
#endif	/* __STACK_EXT_H */
