/*
 * LWE sample linker script
 */
/*
 * Copyright (c) 2008 - 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

ENTRY(_start)

OUTPUT_ARCH(powerpc:common)

/*The memory regions define the guest physical address ranges to be used*/
/* lwe1_pma3 of size 128MB (see in dts) mapped at guest physical 0x90000000
 * is free and available for use
 */
MEMORY
{
	/*Left top 1MB of guest physical memory for loading device tree*/
	ro_mem		:	org = 64M, len = 64M
	data		:	org = 128M, len = 64M
	percpu_data 	:	org = 256M, len = 256K * 7
	percpu_bss_heap_stack	:	org = 512M, len = 64M * 7
	bss		:	org = 1024M, len = 1024M
	stats		:	org = 2048M, len = 256M
	dpa_mem		:	org = 3072M, len = 64M
	whiteboard	:	org = 3328M, len = 64M
}

PHDRS
{
	ro_data PT_LOAD FILEHDR PHDRS FLAGS((4 << 20) | 4 | 1);
	data PT_LOAD FLAGS((4 << 20) | 4 | 2);
	percpu_data PT_LOAD FLAGS(4 | 2); 
	percpu_bss_heap_stack 8 FLAGS(4 | 2);
	bss 8 FLAGS((4 << 20) | 4 | 2);
	stats 8 FLAGS((10 << 20) | 4 | 2);
	dpa_mem 8 FLAGS((4 << 20) | 4 | 2);
	whiteboard 8 FLAGS((4 << 20)| 4 | 2);
}

SECTIONS
{

	/*Top 64K memory is reserved for ELF and program headers*/
	. = 64M + 64K;
	.text . : AT(64M + 64K)
	{
		*(.init)
		*(.text*)
		*(.rodata*)
		*(.got2)
		*(.devtree)
		. = ALIGN(256K);
	} :ro_data

/*	.text_pad . :
	{
		. = ALIGN(64M);
	} :ro_data
*/
	_start_paddr = LOADADDR(.text) + (_start - ADDR(.text));
	_sec_start_paddr = LOADADDR(.text) + (secondary_start - ADDR(.text));

	/DISCARD/ : 
	{
		*(.eh_frame*)
	}

	. = 128M;
	.shared_data . :
	{ 
		*(.data*)
		*(.sdata*)
		*(.apps_shared_data)
	} AT>data :data

	.data_pad . :
	{
		. = ALIGN(64M);
	} AT>data :data

	. = 256M;
	OVERLAY . : AT(256M)
	{
		cpu0_percpu_data {
					_lwe_percpu_data_start = .;
					*(.lwe_percpu);
					_lwe_percpu_data_end = .;
					*(.percpu);
					*(.apps_percpu_data);
					_percpu_data_end = .;
					. = 256K;
				}
		cpu1_percpu_data { . += SIZEOF(cpu0_percpu_data); }
		cpu2_percpu_data { . += SIZEOF(cpu0_percpu_data); }
		cpu3_percpu_data { . += SIZEOF(cpu0_percpu_data); }
		cpu4_percpu_data { . += SIZEOF(cpu0_percpu_data); }
		cpu5_percpu_data { . += SIZEOF(cpu0_percpu_data); }
		cpu6_percpu_data { . += SIZEOF(cpu0_percpu_data); }
		cpu7_percpu_data { }

	} :percpu_data

	_percpu_start = ADDR(cpu0_percpu_data);
	_percpu_end = _percpu_start + SIZEOF(cpu0_percpu_data);

	. = 512M;
	OVERLAY . : AT(512M)
	{
		cpu0_bss_heap_stack {
					*(.apps_percpu_bss);
					_percpu_bss_end = .;
					. = 64M; }
		cpu1_bss_heap_stack { . += SIZEOF(cpu0_bss_heap_stack); }
		cpu2_bss_heap_stack { . += SIZEOF(cpu0_bss_heap_stack); }
		cpu3_bss_heap_stack { . += SIZEOF(cpu0_bss_heap_stack); }
		cpu4_bss_heap_stack { . += SIZEOF(cpu0_bss_heap_stack); }
		cpu5_bss_heap_stack { . += SIZEOF(cpu0_bss_heap_stack); }
		cpu6_bss_heap_stack { . += SIZEOF(cpu0_bss_heap_stack); }
		cpu7_bss_heap_stack {  }
	} :percpu_bss_heap_stack


	_percpu_bss_heap_stack_start = ADDR(cpu0_bss_heap_stack);
	_percpu_bss_heap_stack_end = _percpu_bss_heap_stack_start + SIZEOF(cpu0_bss_heap_stack);
	_percpu_bss_heap_stack_end_paddr = LOADADDR(cpu0_bss_heap_stack) +  SIZEOF(cpu0_bss_heap_stack);
	_percpu_stack_size = 1M;

	. = 1024M;
	.bss . :
	{
		bss_start = .;
		*(.bss)
		*(.sbss*)
		*(COMMON)
		bss_end = .;

		_global_heap_start = .;
		. = 1024M;
		_global_heap_end = .;
	} AT>bss :bss

	. = 2048M;
	.stats .  (NOLOAD):
	{
		*(.stats_data)
		_stats_heap_start = .;
		. = 256M;
		_stats_heap_end = .;
	} AT>stats :stats

	. = 3072M;
	.dpa_mem . : 
	{
		. = . + 64M; 
	} AT>dpa_mem :dpa_mem

	. = 3328M;
	.whiteboard . :
	{
		. = . + 64M;
	} AT>whiteboard :whiteboard

	/*The following two variables must be present so that the dynamic 
	device mapper can get its memory range*/
	. = 4094M;
	_dev_mmap_start = .;
	. = . + 2M;
	_dev_mmap_end = .;
}
