/** 
\file
\brief	intra-partition signaling support.
	This file provides API for communication between cores in a partition.
*/

/*
 * Copyright (c) 2009, 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdbool.h>
#include <io.h>
#include <fsl_hcalls.h>
#include <init.h>
#include <log.h>
#include <errno.h>
#include <misc.h>
#include <core-regs.h>
#include <intra_sgnl_private.h>

__PERCPU struct ip_intr doorbell_intr = { NULL, NULL };

void doorbell_handler(struct trapframe *regs)
{
	/* if handler is not registerd */
	if (doorbell_intr.handler == NULL)
		while (1) ;
	else
		doorbell_intr.handler(regs, doorbell_intr.arg);
}

int register_ip_handler(ip_func handler, void *arg)
{
	if (NULL == handler) {
		LWE_ERROR("ERROR!! NULL intra-partition signaling handler");
		return -1;
	}

	doorbell_intr.handler = handler;
	doorbell_intr.arg = arg;

	return 0;
}

int send_ip_sgnl(uint8_t core_mask)
{
	bool select_core;
	uint8_t dest_core;
	uint32_t target;
	uint32_t online_coremask;
	uint32_t disabled_coremask;

	online_coremask = get_online_core_mask();
	disabled_coremask = ~online_coremask;

	if (core_mask & disabled_coremask) {
		LWE_ERROR("core mask contains invalid core IDs");
		return -EINVAL;
	}

	if (core_mask == online_coremask) {
		target = TARGET_BROADCAST;
		asm volatile ("msgsnd %0"::"r" (target)); /* message type 0 */
		return 0;
	}

	target = 0;
	dest_core = 0x01;
	while (online_coremask) {
		select_core = dest_core & core_mask;
		if (select_core)
			asm volatile ("msgsnd %0"::"r" (target));
		target++;
		dest_core <<= 1;
		online_coremask >>= 1;
	};

	return 0;
}
