/** @file
 * This file provides abort()/ exit()/ _ExitProcess() hooks for libc
 */

/*
 * Copyright (c) 2008 - 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdlib.h>
#include <hv_misc.h>
#include <fsl_hcalls.h>
#include <assert.h>
#include <compiler.h>
#include <hwi.h>
#include <string.h>

void *__real_memcpy(void *dest, const void *src, size_t n);


/** 
 * _ExitProcess() - Stops the current running partition.
 *
 */
void
_ExitProcess(void)
{
	if (fh_partition_stop(0xffffffff))
		assert(0);
}

/** 
 * abort() - Causes ungraceful shutdown of the guest
 * 
 */
void
abort(void)
{
	_ExitProcess();
}

/** exit() - For graceful shutdown of the guest
 *  @param[in] status -- 0 for success and 1 for failure
 *  Currently hypervisor does not support the graceful shutdown of the guest
 *  so exit() is identical to abort() for the time being
 */
void
exit(int status __UNUSED)
{
	_ExitProcess();
}


/* This function implements the ISR safe memory copy routine.
 * We use '-wrap memcpy' linker option. Doing this changes all
 * references of 'memcpy' in code to '__wrap_memcpy' automatically
 */
void *__wrap_memcpy(void *dest, const void *src, size_t n)
{
	if (unlikely(in_irq))
		return isr_memcpy(dest, src, n);	/*Call non-FPU memcpy*/
	else
		return __real_memcpy(dest, src, n);	/*Call normal memcpy*/
}

