/*
\file           
\brief          lwe private malloc routines
*/
/*
 * Copyright (c) 2009, 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <io.h>
#include <malloc_wrapper.h>
#include <log.h>

__PERCPU mspace priv_mspace;
mspace stats_mspace;
mspace sys_mspace;
uint32_t stats_memlock;

mspace
tlmalloc_init(void *start, void *end)
{
	size_t size;
	size = end - start + 1;
	priv_mspace = create_mspace_with_base(start, size, 0);
	if (!priv_mspace) {
		LWE_ERROR("%s: Failed to create mspace", __func__);
		return NULL;
	}

	LWE_INFO("Percpu heap size: %#x bytes", size);
	return priv_mspace;
}

mspace
stats_malloc_init(void *start, void *end)
{
	size_t size;
	size = end - start + 1;
	stats_mspace = create_mspace_with_base(start, size, 0);
	if (!stats_mspace) {
		LWE_ERROR("%s: Failed to create mspace", __func__);
		return NULL;
	}

	LWE_INFO("Stats heap size: %#x bytes", size);
	return stats_mspace;
}

/* Global malloc */
mspace
sys_malloc_init(void *start, void *end)
{
	size_t size;
	size = end - start + 1;
	sys_mspace = create_mspace_with_base(start, size, 1);
	if (!sys_mspace) {
		LWE_ERROR("%s: Failed to create mspace", __func__);
		return NULL;
	}

	LWE_INFO("Global heap size: %#x bytes", size);
	return sys_mspace;
}

__attribute__ ((malloc))
void *
__sys_alloc(size_t size)
{
	return mspace_malloc(sys_mspace, size);
}

void
__sys_free(void *ptr)
{
	mspace_free(sys_mspace, ptr);
}

size_t
__sys_pointer_size(void *ptr)
{
	return mspace_usable_size(ptr);
}
