/**
\file	cpu_usage.c
\brief	Core Usage calculator
*/
/*
 * Copyright (C) 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <cpu_usage.h>
#include <conf.h>
#include <log.h>
#include <compiler.h>
#include <cache.h>

__PERCPU volatile struct cpu_usage_data_t cpu_usage_data
	__ALIGNED(CACHE_LINE_SIZE);

/**
\brief:		This funtion is called only once by the LWE-APP and performs the one-time initializations
\param[in]:	void
\param[out]:	void
\return	:	void
*/
void init_cpu_usage(void)
{
	uint64_t time;
	uint64_t integ_interval, temp;
	uint8_t i;
	uint32_t integ_time;

	integ_interval = timebase_freq*((float)INTEG_DURATION_MS/1000);
	temp = integ_interval;

	for (i = 0; temp != 0; i++)
		temp >>= 1;

	cpu_usage_data.integ_interval = 1<<(i-1);
	cpu_usage_data.integ_interval_shift = i-1;
	cpu_usage_data.threshold = (cpu_usage_data.integ_interval *
					5)/100;

	cpu_usage_data.state = BUSY;
	cpu_usage_data.usage_value = 0;

	time = get_timebase();
	cpu_usage_data.prev = time;
	cpu_usage_data.busy_start = time;

	integ_time = (cpu_usage_data.integ_interval * INTEG_DURATION_MS)/
			integ_interval;

	LWE_INFO("Integration Interval time: %u",
		integ_time);
}

/**
\brief:		This funtion is called when an interrupt occurs
\param[in]:	void
\param[out]:	void
\return	:	void
*/
void core_usage_int_start_handler(void)
{
	if (BUSY != cpu_usage_data.state)
		busy_start();
}
