#!/bin/sh
#
# This script came from FreeBSD (src/sys/kern/genassym.sh, version 1.4).
# The script had no copyright, and the top level FreeBSD copyright
# is below.

# The compilation of software known as FreeBSD is distributed under the
# following terms:
#
# Copyright (C) 1992-2008 The FreeBSD Project. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# The 4.4BSD and 4.4BSD-Lite software is distributed under the following
# terms:
#
# All of the documentation and software included in the 4.4BSD and 4.4BSD-Lite
# Releases is copyrighted by The Regents of the University of California.
#
# Copyright 1979, 1980, 1983, 1986, 1988, 1989, 1991, 1992, 1993, 1994
# 	The Regents of the University of California.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
# This product includes software developed by the University of
# California, Berkeley and its contributors.
# 4. Neither the name of the University nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# The Institute of Electrical and Electronics Engineers and the American
# National Standards Committee X3, on Information Processing Systems have
# given us permission to reprint portions of their documentation.
#
# In the following statement, the phrase ``this text'' refers to portions
# of the system documentation.
#
# Portions of this text are reprinted and reproduced in electronic form in
# the second BSD Networking Software Release, from IEEE Std 1003.1-1988, IEEE
# Standard Portable Operating System Interface for Computer Environments
# (POSIX), copyright C 1988 by the Institute of Electrical and Electronics
# Engineers, Inc.  In the event of any discrepancy between these versions
# and the original IEEE Standard, the original IEEE Standard is the referee
# document.
#
# In the following statement, the phrase ``This material'' refers to portions
# of the system documentation.
#
# This material is reproduced with permission from American National
# Standards Committee X3, on Information Processing Systems.  Computer and
# Business Equipment Manufacturers Association (CBEMA), 311 First St., NW,
# Suite 500, Washington, DC 20001-2178.  The developmental work of
# Programming Language C was completed by the X3J11 Technical Committee.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of the Regents of the University
# of California.
#
#
# NOTE: The copyright of UC Berkeley's Berkeley Software Distribution ("BSD")
# source has been updated.  The copyright addendum may be found at
# ftp://ftp.cs.berkeley.edu/pub/4bsd/README.Impt.License.Change and is
# included below.
#
# July 22, 1999
#
# To All Licensees, Distributors of Any Version of BSD:
#
# As you know, certain of the Berkeley Software Distribution ("BSD") source
# code files require that further distributions of products containing all or
# portions of the software, acknowledge within their advertising materials
# that such products contain software developed by UC Berkeley and its
# contributors.
#
# Specifically, the provision reads:
#
# "     * 3. All advertising materials mentioning features or use of this software
#       *    must display the following acknowledgement:
#       *    This product includes software developed by the University of
#       *    California, Berkeley and its contributors."
#
# Effective immediately, licensees and distributors are no longer required to
# include the acknowledgement within advertising materials.  Accordingly, the
# foregoing paragraph of those BSD Unix files containing it is hereby deleted
# in its entirety.
#
# William Hoskins
# Director, Office of Technology Licensing
# University of California, Berkeley

usage()
{
	echo "usage: genassym [-o outfile] objfile"
	exit 1
}

outfile=/dev/stdout
while getopts "o:" option
do
	case "$option" in
	o)	outfile="$OPTARG";;
	*)	usage;;
	esac
done
shift $(($OPTIND - 1))
case $# in
1)	;;
*)	usage;;
esac

${NM:='nm'} "$1" | ${AWK:='awk'} '
/ C .*sign$/ {
	sign = substr($1, length($1) - 3, 4)
	sub("^0*", "", sign)
	if (sign != "")
		sign = "-"
}
/ C .*w0$/ {
	w0 = substr($1, length($1) - 3, 4)
}
/ C .*w1$/ {
	w1 = substr($1, length($1) - 3, 4)
}
/ C .*w2$/ {
	w2 = substr($1, length($1) - 3, 4)
}
/ C .*w3$/ {
	w3 = substr($1, length($1) - 3, 4)
	w = w3 w2 w1 w0
	sub("^0*", "", w)
	if (w == "")
		w = "0"
	sub("w3$", "", $3)
	# This still has minor problems representing INT_MIN, etc.  E.g.,
	# with 32-bit 2''s complement ints, this prints -0x80000000, which 
	# has the wrong type (unsigned int).
	printf("#define\t%s\t%s0x%s\n", $3, sign, w)
}
' 3>"$outfile" >&3 3>&-
