/** @file
 * head.S
 */

/*
 * Copyright (c) 2008 - 2011 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

.section .init

#include "core-regs.h"
#include "fsl-booke-tlb.h"
#include "percpu.h"
#define WIMGE_MASK 0x1f
/* 32-bit */
#define LONGBYTES 4
#define LOAD lwz
#define STORE stw

/*
 * Assumptions at start are that hardware is in
 * a state as defined by the ePAPR
 *   R3 :  ptr to device tree
 *
 */

.globl _start
_start:

	/* Issue INV_ALL Invalidate on TLB0 */
	li      %r16, 0x04
	tlbivax	0, %r16
	isync
	msync
	nop

/*
 * Use tlbsx to locate the TLB1 entry that maps kernel code
 */
	lis	%r1, (_percpu_bss_heap_stack_end_paddr - FRAMELEN)@ha
	ori	%r1, %r1, (_percpu_bss_heap_stack_end_paddr - FRAMELEN)@l
	
	li 	%r23, fixup_mmu_end - 1f
	bl	fixup_mmu
1:

	lis	%r2, g_cpu_table@ha
	addi	%r2, %r2, g_cpu_table@l
	mtsprg0	%r2

	bl	clear_bss

	lis	%r1, (_percpu_bss_heap_stack_end - FRAMELEN)@ha
	addi	%r1, %r1, (_percpu_bss_heap_stack_end - FRAMELEN)@l
	lis	%r21, 0
	stw 	%r21, 0(%r1)

	bl	ivor_setup
	bl	start

1:	lis	%r3, 0xffff
	mtctr	%r3
2:	bdnz	2b
	b	1b

	.global secondary_start
secondary_start:
	cmpwi   %cr7, %r3, 0
	bne     %cr7, 2f
1:	wait
	b       1b
2:	lis	%r1, (_percpu_bss_heap_stack_end_paddr)@ha
	ori	%r1, %r1, (_percpu_bss_heap_stack_end_paddr)@l

2:	lis	%r5, (FRAMELEN)@ha
	ori	%r5, %r5, (FRAMELEN)@l

	sub 	%r1, %r1, %r5

	li 	%r5, 0
	lis	%r5, (_percpu_bss_heap_stack_end)@ha
	ori	%r5, %r5, (_percpu_bss_heap_stack_end)@l
	
	lis	%r4, (_percpu_bss_heap_stack_start)@ha
	ori	%r4, %r4, (_percpu_bss_heap_stack_start)@l

	sub	%r5, %r5, %r4
	mfspr	%r4, SPR_PIR

	mullw	%r5,%r5,%r4
	add 	%r1,%r1,%r5
	
	li 	%r23, fixup_mmu_end - 1f
	bl	fixup_mmu
1:
	mr	%r2, %r3
	mtsprg0	%r3
	LOAD	%r1, CPU_KSTACK(%r3)

	lis	%r21, 0
	stw 	%r21, 0(%r1)
	bl	ivor_setup
	bl	secondary_init

1:	lis	%r3, 0xffff
	mtctr	%r3
2:	bdnz	2b
	b	1b

fixup_mmu:
	bl	1f
	/* Find entry that maps current address */
1:	mflr	%r19
	mfspr	%r17, SPR_PID
	slwi	%r17, %r17, MAS6_SPID_SHIFT
	mtspr	SPR_MAS6, %r17
	isync
	tlbsx	0, %r19

	/* Copy entry number to r10 */
	mfspr	%r17, SPR_MAS0
	rlwinm	%r10, %r17, 16, 28, 31

	/* Invalidate TLB1, skipping our entry. */

	mfspr	%r17, SPR_TLB1CFG	/* Get number of entries */
	andi.	%r17, %r17, TLBCFG_NENTRY_MASK@l
	li	%r16, 0			/* Start from Entry 0 */

2:	lis	%r15, MAS0_TLBSEL1@h	/* Select TLB1 */
	rlwimi	%r15, %r16, 16, 4, 15
	mtspr	SPR_MAS0, %r15
	isync
	tlbre
	mfspr	%r15, SPR_MAS1
	cmpw	%r16, %r10
	beq	3f
	/* Clear VALID and IPROT bits for other entries */
	rlwinm	%r15, %r15, 0, 2, 31
	mtspr	SPR_MAS1, %r15
	isync
	tlbwe
	isync
	msync
3:	addi	%r16, %r16, 1           /* increment to next index */
	cmpw	%r16, %r17		/* Check if this is the last entry */
	bne	2b

/*
 * Create temporary mapping in the other Address Space
 */
	lis	%r17, MAS0_TLBSEL1@h	/* Select TLB1 */
	rlwimi	%r17, %r10, 16, 12, 15	/* Select our entry */
	mtspr	SPR_MAS0, %r17
	isync
	tlbre				/* Read it in */

	/* Prepare and write temp entry */
	lis	%r17, MAS0_TLBSEL1@h	/* Select TLB1 */
	mfspr	%r18, SPR_TLB1CFG	/* Get number of entries */
	andi.	%r18, %r18, TLBCFG_NENTRY_MASK@l

	cmpwi	%cr7, %r10, 0
	beq	%cr7, j1
	li	%r11, 0
	subi	%r26, %r18, 0x1
	rlwinm	%r26, %r26, 16, 0, 15
	ori	%r26, %r26, 0x1

	b	j2
j1:
	subi	%r26, %r18, 0x1
	mr	%r11, %r26
	subi	%r26, %r26, 0x1
	rlwinm	%r26, %r26, 16, 0, 15

j2:

	rlwimi	%r17, %r11, 16, 12, 15	/* Select temp entry */
	mtspr	SPR_MAS0, %r17
	isync

	mfspr	%r16, SPR_MAS1
	/* AS 1 and setting IMA of 4G for Seondary cores */
	li	%r15, 0x1b
	rlwimi	%r16, %r15, 8, 19, 23 
	mtspr	SPR_MAS1, %r16
	li	%r17, 0
	rlwimi	%r16, %r17, 0, 8, 15	/* Global mapping, TID=0 */

	li	%r17, 0
	ori	%r17, %r17, 0xfff
	mfspr	%r16, SPR_MAS3
	and	%r17, %r17, %r16
	mtspr	SPR_MAS3, %r17
	rlwinm	%r17, %r16, 0, 0, 19

	mfspr %r15, SPR_MAS2
	li %r16, WIMGE_MASK
	andc %r15, %r15, %r16
	li %r16, MAS2_M
	or %r15, %r15, %r16
	mtspr SPR_MAS2, %r15

	isync

	tlbwe
	isync
	msync

	mfmsr	%r16
	ori	%r16, %r16, 0x30	/* Switch to AS 1. */

	mflr	%r15
	addi	%r15, %r15, 4f - 1b	/* Increment to instruction after rfi */
	add	%r15, %r15, %r17
	mr	%r29, %r17

	mtspr	SPR_SRR0, %r15
	mtspr	SPR_SRR1, %r16
	rfi				/* Switch context */

/*
 * Invalidate initial entry
 */
4:	mr	%r22, %r10
	bl	tlb1_inval_entry


/*
 * Setup final mapping in TLB1[BASE_TLB_ENTRY] and switch to it
 */
	mr %r30, %r3
	mr %r31, %r11

	mfspr %r4, SPR_PIR
	cmpwi	%cr1, %r4, 0
	bne- %cr1, j3

	bl copy_devtree
j3:
	mr %r3, %r26
	bl set_memory_tlb

	mr %r3, %r30
	mr %r11, %r31

	/* Switch to the above TLB1[BASE_TLB_ENTRY] mapping */
	lis	%r20, 6f@h
	ori	%r20, %r20, 6f@l
	li	%r21, 0x200

	mtspr   SPR_SRR0, %r20
	mtspr   SPR_SRR1, %r21
	rfi

/*
 * Invalidate temp mapping
 */
6:	mr	%r22, %r11
	bl	tlb1_inval_entry

	lis	%r17, fixup_mmu_end@h
	ori	%r17, %r17, fixup_mmu_end@l
	subf	%r23, %r23, %r17
	mtlr	%r23
fixup_mmu_end:
	blr

tlb1_inval_entry:
	lis	%r17, MAS0_TLBSEL1@h	/* Select TLB1 */
	rlwimi	%r17, %r22, 16, 12, 15	/* Select our entry */
	mtspr	SPR_MAS0, %r17
	isync
	tlbre				/* Read it in */

	li	%r16, 0
	mtspr	SPR_MAS1, %r16
	isync
	tlbwe
	isync
	msync
	blr

ivor_setup:
	/* Set base address of interrupt handler routines */
	lis	%r21, interrupt_vector_base@h
	mtspr	SPR_IVPR, %r21

	/* Assign interrupt handler routines offsets */
	li	%r21, int_critical_input@l
	mtspr	SPR_IVOR0, %r21
	li	%r21, int_machine_check@l
	mtspr	SPR_IVOR1, %r21
	li	%r21, int_data_storage@l
	mtspr	SPR_IVOR2, %r21
	li	%r21, int_instr_storage@l
	mtspr	SPR_IVOR3, %r21
	li	%r21, int_external_input@l
	mtspr	SPR_IVOR4, %r21
	li	%r21, int_alignment@l
	mtspr	SPR_IVOR5, %r21
	li	%r21, int_program@l
	mtspr	SPR_IVOR6, %r21
	li	%r21, int_fpunavail@l
	mtspr	SPR_IVOR7, %r21
	li	%r21, int_syscall@l
	mtspr	SPR_IVOR8, %r21
	li	%r21, int_decrementer@l
	mtspr	SPR_IVOR10, %r21
	li	%r21, int_fixed_interval_timer@l
	mtspr	SPR_IVOR11, %r21
	li	%r21, int_watchdog@l
	mtspr	SPR_IVOR12, %r21
	li	%r21, int_data_tlb_error@l
	mtspr	SPR_IVOR13, %r21
	li	%r21, int_inst_tlb_error@l
	mtspr	SPR_IVOR14, %r21
	li	%r21, int_debug@l
	mtspr	SPR_IVOR15, %r21
	li	%r21, int_perf_mon@l
	mtspr	SPR_IVOR35, %r21
	li	%r21, int_doorbell@l
	mtspr	SPR_IVOR36, %r21
	li	%r21, int_doorbell_critical@l
	mtspr	SPR_IVOR37, %r21

	blr

clear_bss:
	lis	%r23, (bss_start - 1)@h
	lis	%r24, (bss_end - 1)@h
	ori	%r23, %r23, (bss_start - 1)@l
	ori	%r24, %r24, (bss_end - 1)@l

	sub	%r25, %r24, %r23
	mtctr	%r25

	li	%r26, 0
1:	stbu	%r26, 1(%r23)
	bdnz	1b

	blr
