/** @file
 * Hello World Test case
 */

/*
 * Copyright (c) 2008, 2009 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "init.h"
#include "misc.h"
#include "compat.h"
#include "fsl_bman.h"

extern void exit(int);

static const struct bman_pool_params params = {
	.bpid = 17,
};
static const struct bm_buffer buffers[2] = {
	{ .hi = 0xdead, .lo = 0xbeefabba },
	{ .hi = 0xf00d, .lo = 0x12345678 }
};
static __PERCPU struct bm_buffer output[2];

static void delay_1000(void)
{
	u64 now, then = mfatb();
	do {
		now = mfatb();
	} while (now < (then + 1000));
}

int main(__always_unused int argc, __always_unused char **argv)
{
	int num_retries = 5;
	struct bman_pool *pool = bman_new_pool(&params);
	pr_info("Bman sanity\n");
	if (!pool)
		panic("bman_new_pool() failed");
	pr_info("Got pool\n");
	if (bman_release(pool, buffers, 2, 0))
		panic("bman_release() failed");
	pr_info("Released 2 bufs\n");
	/* The release command is posted, and we need to be sure it has been
	 * processed by Bman before we try our acquire. We give it plenty of
	 * time before declaring a problem. 5 retries, with 1000 cycle delays
	 * each time is well in excess of expected worst-case h/w performance,
	 * but still amounts to less than 4 microseconds @1.5Ghz. */
	while (!bman_rcr_is_empty()) {
		if (!(num_retries--))
			panic("bman_release() isn't syncing");
		delay_1000();
	}
	if (bman_acquire(pool, output, 2, 0) != 2)
		panic("bman_acquire() failed");
	pr_info("Acuired 2 bufs\n");
	pr_info("  a = 0x%04x %08x\n", output[0].hi, output[0].lo);
	pr_info("  b = 0x%04x %08x\n", output[1].hi, output[1].lo);
	bman_free_pool(pool);
	pr_info("Freed pool\n");
	return 0;
}
