/** @file
 * Hello World Test case for Alternate programming Model
 * Here global variables are treated as percpu variables
 * If application wants to share any data amongst cores the
 * data should be put in .app_shared_data section.
 */

/*
 * Copyright (c) 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdlib.h>
#include "init.h"
#include "misc.h"
#include <log.h>
#include <compiler.h>
#include <barrier.h>

#define __SHARED __attribute__ ((section(".apps_shared_data")))

volatile int global_percpu_var = 1;
__SHARED int shared_var = 1;
__SHARED struct lwe_barrier core_barrier;
__SHARED unsigned int lock;

int main(int argc __UNUSED, char **argv __UNUSED)
{
	LWE_INFO("Hello World");
	LWE_INFO("The lwe id is %u", get_lwe_id());

	LWE_INFO("Here we are using alternate programming model where \
		  the global variables will be treated as percpu\n");

	LWE_INFO("Test 1 : Initial value of global_percpu_var is %d\n",
						global_percpu_var);
	barrier_init(&core_barrier, get_online_core_mask());
	if (lwe_instance == 0) {
		LWE_INFO("Lwe Instance %d incrementing global_percpu_var",
							get_lwe_id());
		global_percpu_var++;
	}

	LWE_INFO("All the cores waiting for lwe_instance 0 to increment \
		  the global_percpu_var\n");
	barrier_sync(&core_barrier);
	LWE_INFO("Value of global_percpu_var is %d\n", global_percpu_var);


	LWE_INFO("Test 2:- Initial value of shared variable %d\n", shared_var);
	spin_lock(&lock);
	shared_var++;
	spin_unlock(&lock);

	barrier_sync(&core_barrier);
	LWE_INFO("Final value of shared variable %d\n", shared_var);

	barrier_destroy(&core_barrier);
	return 0;
}
