/** @file
 * Hello World C++ Test case
 */

/*
 * Copyright (c) 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <init.h>
#include <swi.h>
#include <lwe_atomic.h>
#include <statistics.h>
#include <ll_struct.h>
#include <listmgr.h>
#include <conf.h>
#include <compiler.h>
#include <stack.h>
#include <libfdt_env.h>
#include <sg_api.h>
#include <checksums.h>
#include <intra_sgnl.h>
#include <perf_mon.h>
#include <timebase.h>
#include <timer_wheel.h>
#include <barrier.h>
#include <exception.h>
#include <hwi.h>
#include <nmi.h>
#include <rcu_lock.h>
#include <rbtree.h>
#include <rwlock.h>
#include <wdog.h>
#include <lwe_malloc.h>
#include <shell.h>
#include <hash.h>
#include <fm_types.h>
#include <crc64.h>
#include <misc.h>
#include <log.h>
#include <dpa_alloc.h>
#include <mmap.h>
#include <cache.h>
#include <endian.h>
#include <assym.h>
#include <hv_misc.h>
#include <lwe_io.h>
#include <malloc.h>
#include <fsl_hcalls.h>
#include <io.h>
#include <hv_dep.h>
#include <bitops.h>
#include <fsl-booke-tlb.h>
#include <trapframe.h>
#include <trap_booke.h>
#include <lerrors.h>
#include <core-regs.h>
#include <types.h>

// CAAM
#include <dcl.h>
#include <desc.h>
#include <pdb.h>

#include <stdlib.h>

#include <compat.h>
#include <fsl_qman.h>
#include <fsl_bman.h>
#include <fsl_pme.h>

// dummy classes to demonstrate the following language features:
//  - classes
//  - inheritance
//  - virtual base classes

class base {
	struct lwe_barrier *my_dummy_barrier;
public:
	base()
	{
		my_dummy_barrier = (struct lwe_barrier *)
				malloc(sizeof(struct lwe_barrier));
		barrier_init(my_dummy_barrier, 0xf0000000);
	}
	virtual ~base()
	{ barrier_destroy(my_dummy_barrier); free(my_dummy_barrier); }
	virtual void dummy_virt_func(void) = 0;
};

struct derived : public virtual base {
	void dummy_virt_func(void)
	{ LWE_INFO("derived:dummy_virt_func"); }
};

extern "C" void __cxa_pure_virtual(void)
{
	// call to a pure virtual function happened ...
	// wow, should never happen ... stop
	while(1)
	{}
}

void *operator new(unsigned int size)
{
	// implemented temporarily in terms of malloc
	return malloc(size);
}

void operator delete(void *ptr)
{
	// implemented temporarily in terms of free
	free(ptr);
}

#define my_mfspr(reg) \
({ \
	register_t ret; \
	asm volatile("mfspr %0, %1" : "=r" (ret) : "i" (reg) : "memory"); \
	ret; \
})
static inline uint64_t
my_get_timebase(void)
{
	uint32_t hi, lo, chk;

	/*
	 * To make sure that there is no carry over
	 * between checking of TBU and TBL
	 */
	do {
		hi = my_mfspr(SPR_ATBU);
		lo = my_mfspr(SPR_ATBL);
		chk = my_mfspr(SPR_ATBU);
	} while (unlikely(hi != chk));

	return (uint64_t) hi << 32 | (uint64_t) lo;
}

/*************/
/* constants */
/*************/

#define NUM_IGNORE	18
#define NUM_CPUS	7
#undef MODEL_CHKPT
#undef TEST_FD

/*************************************/
/* Predeclarations (eg. for fq_base) */
/*************************************/

static enum qman_cb_dqrr_result cb_dqrr(struct qman_portal *,
					struct qman_fq *,
					const struct qm_dqrr_entry *);
static void cb_ern(struct qman_portal *, struct qman_fq *,
			const struct qm_mr_entry *);
static void cb_dc_ern(struct qman_portal *, struct qman_fq *,
			const struct qm_mr_entry *);

/***************/
/* global vars */
/***************/

static __PERCPU struct qm_fd fd;
static __PERCPU struct qm_fd fd_dq;
static __PERCPU struct qman_fq fq_base;

struct waker {
	volatile int kick;
} __cacheline_aligned;
static struct waker waker[NUM_CPUS];

static __PERCPU u64 eq_capture[2] ____cacheline_aligned;
static __PERCPU u64 dq_capture[2] ____cacheline_aligned;
static __PERCPU u32 test_frames;
static __PERCPU u32 test_start;
static __PERCPU u32 dq_count;
static __PERCPU u32 eq_jam;
static __PERCPU int sdqcr_complete;

/* This is our test array (only used by cpu 0) */
struct test {
	int num_cpus;
	u32 num_enqueues;
};
static const struct test tests[] = {
	{ 1, 256},
	{ 2, 256},
	{ 4, 256},
	{ 7, 256},
	{ 0, 0}
};

/* Helpers for initialising and "incrementing" a frame descriptor */
static void fd_init(struct qm_fd *__fd)
{
	__fd->addr_hi = 0xab;		/* high 8-bits */
	__fd->addr_lo = 0xdeadbeef;	/* low 32-bits */
	__fd->format = qm_fd_contig_big;
	__fd->length29 = 7;
	__fd->cmd = 0xfeedf00d;
}

#ifdef TEST_FD
static void fd_inc(struct qm_fd *__fd)
{
	__fd->addr_lo++;
	__fd->addr_hi--;
	__fd->cmd++;
}

/* The only part of the 'fd' we can't memcmp() is the ppid */
static int fd_cmp(const struct qm_fd *a, const struct qm_fd *b)
{
	int r = a->addr_hi - b->addr_hi;
	if (!r)
		r = a->addr_lo - b->addr_lo;
	if (!r)
		r = a->format - b->format;
	if (!r)
		r = a->opaque - b->opaque;
	if (!r)
		r = a->cmd - b->cmd;
	return r;
}
#endif

static int elapse(u64 *a)
{
	return (int)(a[1] - a[0]);
}

static int avg(u64 *a)
{
	int e = elapse(a);
	int d = test_start;
	return (e + (d / 2)) / d;
}

/********/
/* test */
/********/

static void do_enqueues(struct qman_fq *fq)
{
	unsigned int loop = test_frames;
	dcbt_rw(eq_capture);
	while (loop) {
		int err;
		if (loop == test_start)
			eq_capture[0] = my_get_timebase();
retry:
		err = qman_enqueue(fq, &fd, 0);
		if (err) {
			eq_jam++;
			barrier();
			goto retry;
		}
#ifdef TEST_FD
		fd_inc(&fd);
#endif
		loop--;
	}
	eq_capture[1] = my_get_timebase();
}

static enum qman_cb_dqrr_result cb_dqrr(struct qman_portal *p,
					struct qman_fq *fq,
					const struct qm_dqrr_entry *dq)
{
#ifdef TEST_FD
	if (fd_cmp(&fd_dq, &dq->fd)) {
		pr_err("BADNESS: dequeued frame doesn't match;\n");
		pr_err("Got:\n");
		hexdump(&dq->fd, sizeof(dq->fd));
		pr_err("Expected:\n");
		hexdump(&fd_dq, sizeof(fd_dq));
		BUG();
	}
	fd_inc(&fd_dq);
#endif
	if (dq_count-- == test_start)
		dq_capture[0] = my_get_timebase();
	else if (!dq_count) {
		dq_capture[1] = my_get_timebase();
		BUG_ON(!(dq->stat & QM_DQRR_STAT_FQ_EMPTY));
		sdqcr_complete = 1;
	}
	return qman_cb_dqrr_consume;
}

static void cb_ern(struct qman_portal *p, struct qman_fq *fq,
			const struct qm_mr_entry *msg)
{
	panic("cb_ern() unimplemented");
}

static void cb_dc_ern(struct qman_portal *p, struct qman_fq *fq,
			const struct qm_mr_entry *msg)
{
	panic("cb_dc_ern() unimplemented");
}

#ifdef MODEL_CHKPT
static volatile int done_print;
static spinlock_t bringup_lock = SPIN_LOCK_UNLOCKED;
#endif
static unsigned long cpus_ready = 0;

/* At the beginning of each test, the cpus all call this routine. The secondary
 * cores signal their readyness for the next test by incrementing "ready", the
 * primary core polls until "ready" hits the desired total. (Ie. except for
 * one-off writes from secondary cores, "ready" stays cache-local to cpu 0.)
 * After incrementing "ready", the secondary cores wait for the go-ahead by
 * polling on the percpu "waker" signal, which cpu 0 sets once everyone is
 * ready. (Ie. except for one-off writes from cpu 0, the "waker" values stay
 * cache-local to their respective cpus.)
 *
 * The idea behind all this is to avoid bursts of coherency noise when tests are
 * starting up or finishing, as these would contaminate the results. This
 * signalling involves some coherency action, but only for the one-off writes,
 * the important thing is that the polling should stay cache-local. */
static void sync_cpus(int whoami)
{
	unsigned int dumbloop;
	atomic_inc(&cpus_ready);
	if (whoami) {
		/* secondary cpus wait for the "go" */
		while (!waker[whoami].kick)
			;
		waker[whoami].kick = 0;
	} else {
		/* cpu 0 waits for the others to be ready */
		while (atomic_read(&cpus_ready) < NUM_CPUS)
			;
		atomic_set(&cpus_ready, 0);
#ifndef MODEL_CHKPT
		dumbloop = 0;
		while (dumbloop < 0x100000)
			dumbloop++;
#endif
		dumbloop = 1;
		while (dumbloop < NUM_CPUS)
			waker[dumbloop++].kick = 1;
	}
}

static int test_foo(void)
{
	struct qm_mcc_initfq initfq;
	int res, whoami = get_lwe_id();
	unsigned int flags;
	struct qman_fq *fq = &fq_base;
	const struct test *test = &tests[0];

	fq_base.cb.dqrr = cb_dqrr;
	fq_base.cb.ern = cb_ern;
	fq_base.cb.dc_ern = cb_dc_ern;
	sync_cpus(whoami);
#ifdef MODEL_CHKPT
	/* Do a dance so that we can checkpoint when we see "speed starting",
	 * know that no cpu has yet started testing, and that post-checkpoint we
	 * do no more console activity on any cpu. */
	spin_lock(&bringup_lock);
	if (!done_print) {
		pr_info("%d: speed starting\n", get_lwe_id());
		done_print = 1;
	}
	spin_unlock(&bringup_lock);
#endif
	fd_init(&fd);
	fd_init(&fd_dq);

	while (test->num_cpus) {
		int doIrun = (whoami < test->num_cpus) ? 1 : 0;

		test_frames = test->num_enqueues;
		test_start = test->num_enqueues - NUM_IGNORE;
		dq_count = test->num_enqueues;
		eq_jam = 0;

		if (doIrun) {
			/* Initialise (parked) FQ */
			if (qman_create_fq(0, QMAN_FQ_FLAG_DYNAMIC_FQID, fq))
				panic("qman_create_fq() failed\n");
			memset(&initfq, 0, sizeof(initfq));
			initfq.we_mask = QM_INITFQ_WE_FQCTRL |
					QM_INITFQ_WE_CONTEXTA;
			initfq.fqd.fq_ctrl = QM_FQCTRL_CTXASTASHING;
			initfq.fqd.context_a.stashing.context_cl =
					(sizeof(*fq) + 63) / 64;

			if (qman_init_fq(fq, QMAN_INITFQ_FLAG_LOCAL, &initfq))
				panic("qman_init_fq() failed\n");
		}

		sync_cpus(whoami);

		if (doIrun) {
			do_enqueues(fq);
			dcbt_rw(dq_count);
			dcbt_rw(dq_capture);
			if (qman_schedule_fq(fq))
				panic("qman_schedule_fq() failed\n");
			wait_event(nothing, sdqcr_complete);
			sdqcr_complete = 0;
		}

		sync_cpus(whoami);

		if (doIrun) {
			u32 flags;
			if (qman_retire_fq(fq, &flags))
				panic("qman_retire_fq() failed\n");
			BUG_ON(flags & (QMAN_FQ_STATE_CHANGING |
				QMAN_FQ_STATE_NE | QMAN_FQ_STATE_ORL));
			if (qman_oos_fq(fq))
				panic("qman_oos_fq() failed\n");
			pr_info("test {%d,%d} eq %d, dq %d, eq_jam %d\n",
				test->num_cpus, test->num_enqueues,
				avg(eq_capture), avg(dq_capture), eq_jam);
		}
		test++;
	}

	return 0;
}

int main(int argc, char **argv)
{
	base *bptr = new derived;
	bptr->dummy_virt_func();
	LWE_INFO("Hello World");
	LWE_INFO("The lwe id is %u", get_lwe_id());

	LWE_INFO("sizeof(struct bm_mcc_acquire)=%d\n", sizeof(struct bm_mcc_acquire));
	LWE_INFO("sizeof(struct bm_mcc_query)=%d\n", sizeof(struct bm_mcc_query));
	LWE_INFO("sizeof(struct bm_mc_command)=%d\n", sizeof(struct bm_mc_command));
	LWE_INFO("sizeof(struct qm_mcc_initfq)=%d\n", sizeof(struct qm_mcc_initfq));
	LWE_INFO("sizeof(struct qm_mcc_initcgr)=%d\n", sizeof(struct qm_mcc_initcgr));
	LWE_INFO("sizeof(struct qm_mcc_querywq)=%d\n", sizeof(struct qm_mcc_querywq));
	LWE_INFO("sizeof(struct qm_mc_command)=%d\n", sizeof(struct qm_mc_command));
	LWE_INFO("sizeof(struct qm_mcr_initfq)=%d\n", sizeof(struct qm_mcr_initfq));
	LWE_INFO("sizeof(struct qm_mcr_queryfq)=%d\n", sizeof(struct qm_mcr_queryfq));
	LWE_INFO("sizeof(struct qm_mcr_queryfq_np)=%d\n", sizeof(struct qm_mcr_queryfq_np));
	LWE_INFO("sizeof(struct qm_mcr_querycgr)=%d\n", sizeof(struct qm_mcr_querycgr));
	LWE_INFO("sizeof(struct qm_mcr_querycongestion)=%d\n", sizeof(struct qm_mcr_querycongestion));
	LWE_INFO("sizeof(struct qm_mcr_querywq)=%d\n", sizeof(struct qm_mcr_querywq));
	LWE_INFO("sizeof(struct qm_mc_result)=%d\n", sizeof(struct qm_mc_result));
	test_foo();
	return 0;
}

