/* Copyright (c) 2009-2010 Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Freescale Semiconductor nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 *
 * ALTERNATIVELY, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") as published by the Free Software
 * Foundation, either version 2 of that License or (at your option) any
 * later version.
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "init.h"
#include "misc.h"
#include "compat.h"
#include "fsl_pme.h"

/* Define this to produce console output */
#undef CONFIG_LOG

#ifdef CONFIG_LOG
#define LOG(a) pr_info a
#else
#define LOG(a) do { ; } while (0)
#endif

/* Define this to get success-path hexdumps */
#undef CONFIG_VERBOSE

#define NUM_CPUS 7
#ifdef CONFIG_LOG
#define PROGRESS 121
#define NUM_LOOPS 120
#else
#define PROGRESS 100
#define NUM_LOOPS 300
#endif

static u8 __scan_data[] = {
	0x41,0x42,0x43,0x44,0x45
};

static u8 __result_data[] = {
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00
};

static u8 __scan_result_direct_mode_inc_mode[] = {
	0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00
};

static u8 __fl_ctx_exp[] = {
	0x88, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xe0,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00
};

/* same again with 'sos' bit cleared */
static u8 __fl_ctx_exp_post_scan[] = {
	0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xe0,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00
};

#define to_malloc(name) \
do { \
	name = malloc(sizeof(__##name)); \
	BUG_ON(!name); \
	memcpy(name, &__##name, sizeof(__##name)); \
} while(0)

struct ctrl_token {
	struct pme_ctx_ctrl_token token;
	struct qm_fd result_fd;
	int done;
};

static void ctrl_cb(__always_unused struct pme_ctx *ctx, const struct qm_fd *fd,
				struct pme_ctx_ctrl_token *token)
{
	struct ctrl_token *t = (void *)token;
	memcpy(&t->result_fd, fd, sizeof(*fd));
	t->done = 1;
}

struct scan_ctx {
	struct pme_ctx base_ctx;
	struct qm_fd result_fd;
};

static __PERCPU DECLARE_COMPLETION(scan_comp);

static void scan_cb(struct pme_ctx *ctx, const struct qm_fd *fd,
		__always_unused struct pme_ctx_token *ctx_token)
{
	struct scan_ctx *my_ctx = (struct scan_ctx *)ctx;
#ifdef CONFIG_VERBOSE
	pr_info("st: scan_cb() invoked, fd;!\r\n");
	hexdump(fd, sizeof(*fd));
#endif
	memcpy(&my_ctx->result_fd, fd, sizeof(*fd));
	complete(&scan_comp);
}

struct scan_ctx *alloc_scan_ctx(void)
{
	struct scan_ctx *ret = memalign(64, sizeof(struct scan_ctx) + 64);
	if (ret) {
		memset(ret, 0, sizeof(struct scan_ctx));
		ret->base_ctx.cb = scan_cb;
	}
	return ret;
}

int main(__always_unused int argc, __always_unused char *argv[])
{
	struct pme_flow flow;
	struct pme_flow rb_flow;
	struct ctrl_token ctoken = {
		.token = {
			.cb = ctrl_cb
		}
	};
	int ctx_cl = (sizeof(struct scan_ctx) + 63) / 64;
	struct qm_fqd_stashing stashing = {
		.data_cl = 3,
		.context_cl = (ctx_cl > 3) ? 3 : ctx_cl
	};
	struct qm_fd fd;
	struct qm_sg_entry *sg_table;
	int ret;
	enum pme_status status;
	struct pme_ctx_token token;
	u8 *scan_result, *scan_data, *result_data;
	u8 *scan_result_direct_mode_inc_mode;
	u8 *fl_ctx_exp, *fl_ctx_exp_post_scan;
	struct scan_ctx *a_scan_ctx;
	u32 scan_result_size;
	int loop = 0, progress = 0;

	if (get_lwe_id() >= NUM_CPUS)
		return 0;
	pr_info("Stashing (# cachlines): data=%d, context=%d\n",
		stashing.data_cl, stashing.context_cl);

	sg_table = memalign(sizeof(struct qm_sg_entry),
			2 * sizeof(struct qm_sg_entry));
	a_scan_ctx = alloc_scan_ctx();
	BUG_ON(!sg_table || !a_scan_ctx);
	to_malloc(scan_data);
	to_malloc(result_data);
	to_malloc(scan_result_direct_mode_inc_mode);
	to_malloc(fl_ctx_exp);
	to_malloc(fl_ctx_exp_post_scan);

	scan_result = scan_result_direct_mode_inc_mode;
	scan_result_size = sizeof(__scan_result_direct_mode_inc_mode);

loop_start:
	/**********************************************************************/
	/**********************************************************************/
	/********************* Direct Mode ************************************/
	/**********************************************************************/
	/**********************************************************************/
	ret = pme_ctx_init(&a_scan_ctx->base_ctx,
			PME_CTX_FLAG_DIRECT | PME_CTX_FLAG_LOCAL,
			0, 4, 4, 0, &stashing);
	BUG_ON(ret);
	/* enable the context */
	pme_ctx_enable(&a_scan_ctx->base_ctx);

	/* Do a pre-built output, scan with match test */
	/* Build a frame descriptor */
	memset(&fd, 0, sizeof(struct qm_fd));
	memset(sg_table, 0, 2 * sizeof(struct qm_sg_entry));

	/* build the result */
	sg_table[0].addr_lo = (u32)dma_map_single(NULL, result_data,
			sizeof(__result_data), DMA_TO_DEVICE);
	sg_table[0].length = sizeof(__result_data);
	sg_table[1].addr_lo = (u32)dma_map_single(NULL, scan_data,
			sizeof(__scan_data), DMA_TO_DEVICE);
	sg_table[1].length = sizeof(__scan_data);
	sg_table[1].final = 1;

	fd._format2 = qm_fd_compound;
	fd.addr_lo = (u32)dma_map_single(NULL, sg_table,
				2 * sizeof(struct qm_sg_entry), DMA_TO_DEVICE);

	LOG(("st: Send scan request\r\n"));
#ifdef CONFIG_VERBOSE
	pr_info("sg_table=%p, array entries 0 and 1;\r\n", sg_table);
	hexdump(sg_table, 2 * sizeof(struct qm_sg_entry));
	pr_info("FD;\r\n");
	hexdump(&fd, sizeof(fd));
#endif
	ret = pme_ctx_scan(&a_scan_ctx->base_ctx, 0, &fd,
		PME_SCAN_ARGS(PME_CMD_SCAN_SR | PME_CMD_SCAN_E, 0, 0xff00),
		&token);
	LOG(("st: Response scan %d\r\n", ret));
	wait_for_completion(&scan_comp);

	LOG(("st: Status is fd.status %x\r\n",
			a_scan_ctx->result_fd.status));

	status = pme_fd_res_status(&a_scan_ctx->result_fd);
	if (status) {
		pr_info("st: Scan status failed %d\r\n", status);
		BUG_ON(1);
	}
	if (memcmp(scan_result,result_data, scan_result_size) != 0) {
		pr_info("st: Scan result expected;\r\n");
		hexdump(scan_result, scan_result_size);
		pr_info("st: Received...\r\n");
		hexdump(result_data, sizeof(__result_data));
		pr_info("And the table...\r\n");
		hexdump(sg_table, 2 * sizeof(struct qm_sg_entry));
		BUG_ON(1);
	}
	/* Test truncation test */

	/* Build a frame descriptor */
	memset(&fd, 0, sizeof(struct qm_fd));
	fd.length20 = sizeof(__scan_data);
	fd.addr_lo = (u32)dma_map_single(NULL, scan_data, sizeof(__scan_data),
			DMA_TO_DEVICE);

	LOG(("st: Send scan request\r\n"));

	ret = pme_ctx_scan(&a_scan_ctx->base_ctx, 0, &fd,
		PME_SCAN_ARGS(PME_CMD_SCAN_SR | PME_CMD_SCAN_E, 0, 0xff00),
		&token);

	LOG(("st: Response scan %d\r\n", ret));
	wait_for_completion(&scan_comp);

	LOG(("st: Status is fd.status %x\r\n",
			a_scan_ctx->result_fd.status));

	status = pme_fd_res_status(&a_scan_ctx->result_fd);
	LOG(("st: Scan status %x\r\n", status));

	/* Check the response...expect truncation bit to be set */
	if (!(pme_fd_res_flags(&a_scan_ctx->result_fd) & PME_STATUS_TRUNCATED)) {
		pr_info("st: Scan result failed...expected trunc\r\n");
		BUG_ON(1);
	}
	LOG(("st: Simple scan test Passed\r\n"));

	/* Disable */
	ret = pme_ctx_disable(&a_scan_ctx->base_ctx, 0, NULL);
	BUG_ON(ret);
	pme_ctx_finish(&a_scan_ctx->base_ctx);

	/**********************************************************************/
	/**********************************************************************/
	/*********************** Flow Mode ************************************/
	/**********************************************************************/
	/**********************************************************************/
	LOG(("st: Start Flow Mode Test\r\n"));

	pme_sw_flow_init(&flow);
	ret = pme_ctx_init(&a_scan_ctx->base_ctx,
		PME_CTX_FLAG_LOCAL, 0, 4, 4, 0, &stashing);
	BUG_ON(ret);

	/* enable the context */
	pme_ctx_enable(&a_scan_ctx->base_ctx);
	LOG(("st: Context Enabled\r\n"));

	/* Set up a default flow context */
	ctoken.done = 0;
	ret = pme_ctx_ctrl_update_flow(&a_scan_ctx->base_ctx, PME_CMD_FCW_ALL,
				(struct pme_flow *)fl_ctx_exp, &ctoken.token);
	BUG_ON(ret);
	wait_event(NULL, ctoken.done == 1);
	/* Check for error */
	status = pme_fd_res_status(&ctoken.result_fd);
	if (status) {
		pr_info("Update Flow failed %d\r\n", status);
		BUG_ON(1);
	}
	LOG(("st: Flashed initial flow context\r\n"));

	/* read back flow settings */
	memset(&rb_flow, 0, sizeof(struct pme_flow));
#ifdef CONFIG_VERBOSE
	pr_info("st: Initial rb_flow\r\n");
	hexdump(&rb_flow, sizeof(rb_flow));
#endif
	ctoken.done = 0;
	ret = pme_ctx_ctrl_read_flow(&a_scan_ctx->base_ctx, 0,
					&rb_flow, &ctoken.token);
	BUG_ON(ret);
	wait_event(NULL, ctoken.done == 1);
	/* Check for error */
	status = pme_fd_res_status(&ctoken.result_fd);
	if (status) {
		pr_info("Read Flow failed %d\r\n", status);
		BUG_ON(1);
	}
	if (memcmp(&rb_flow, fl_ctx_exp, sizeof(rb_flow)) != 0) {
		pr_info("st: Flow Context Read FAIL\r\n");
		pr_info("st: Expected\r\n");
		hexdump(fl_ctx_exp, sizeof(*fl_ctx_exp));
		pr_info("st: Received...\r\n");
		hexdump(&rb_flow, sizeof(rb_flow));
		BUG_ON(1);
	}
	LOG(("st: Flow Context Read OK\r\n"));

	/* Test that we survive a disable/enable */
	ctoken.done = 0;
	ret = pme_ctx_disable(&a_scan_ctx->base_ctx, 0, &ctoken.token);
	BUG_ON(ret < 1);
	wait_event(NULL, ctoken.done == 1);
	ret = pme_ctx_enable(&a_scan_ctx->base_ctx);
	BUG_ON(ret);

	/* Do a pre-built output, scan with match test */
	/* Build a frame descriptor */
	memset(&fd, 0, sizeof(struct qm_fd));
	memset(sg_table, 0, 2 * sizeof(struct qm_sg_entry));

	/* build the result */
	sg_table[0].addr_lo = (u32)dma_map_single(NULL, result_data,
			sizeof(__result_data), DMA_TO_DEVICE);
	sg_table[0].length = sizeof(__result_data);
	sg_table[1].addr_lo = (u32)dma_map_single(NULL, scan_data,
			sizeof(__scan_data), DMA_TO_DEVICE);
	sg_table[1].length = sizeof(__scan_data);
	sg_table[1].final = 1;

	fd._format2 = qm_fd_compound;
	fd.addr_lo = (u32)dma_map_single(NULL, sg_table, sizeof(sg_table),
			DMA_TO_DEVICE);
	LOG(("st: Send scan request\r\n"));
	ret = pme_ctx_scan(&a_scan_ctx->base_ctx, 0, &fd,
		PME_SCAN_ARGS(PME_CMD_SCAN_SR | PME_CMD_SCAN_E, 0, 0xff00),
		&token);
	LOG(("st: Response scan %d\r\n", ret));
	wait_for_completion(&scan_comp);
	LOG(("st: Status is fd.status %x\r\n",
			a_scan_ctx->result_fd.status));
	status = pme_fd_res_status(&a_scan_ctx->result_fd);
	if (status) {
		pr_info("st: Scan status failed %d\r\n", status);
		BUG_ON(1);
	}
	if (memcmp(scan_result,result_data, scan_result_size) != 0) {
		pr_info("st: Scan result not expected, Expected\r\n");
		hexdump(scan_result, scan_result_size);
		pr_info("st: Received...\r\n");
		hexdump(result_data, sizeof(__result_data));
		BUG_ON(1);
	}

	ctoken.done = 0;
	ret = pme_ctx_disable(&a_scan_ctx->base_ctx, 0, &ctoken.token);
	BUG_ON(ret < 1);
	wait_event(NULL, ctoken.done == 1);
	ret = pme_ctx_enable(&a_scan_ctx->base_ctx);
	BUG_ON(ret);

	/* read back flow settings */
	ctoken.done = 0;
	ret = pme_ctx_ctrl_read_flow(&a_scan_ctx->base_ctx, 0, &rb_flow,
					&ctoken.token);
	BUG_ON(ret);
	wait_event(NULL, ctoken.done == 1);
	/* Check for error */
	status = pme_fd_res_status(&ctoken.result_fd);
	if (status) {
		pr_info("Read Flow failed %d\r\n", status);
		BUG_ON(1);
	}
	if (memcmp(&rb_flow, fl_ctx_exp_post_scan, sizeof(rb_flow)) != 0) {
		pr_info("st: Flow Context Read FAIL\r\n");
		pr_info("st: Expected\r\n");
		hexdump(fl_ctx_exp_post_scan, sizeof(__fl_ctx_exp_post_scan));
		pr_info("st: Received\r\n");
		hexdump(&rb_flow, sizeof(rb_flow));
		BUG_ON(1);
	}
	LOG(("st: Flow Context Read OK\r\n"));

	/* Truncation test (with flush) */
	/* Build a frame descriptor */
	memset(&fd, 0, sizeof(struct qm_fd));
	fd.length20 = sizeof(__scan_data);
	fd.addr_lo = (u32)dma_map_single(NULL, scan_data, sizeof(__scan_data),
			DMA_TO_DEVICE);
	LOG(("st: Send scan request\r\n"));
	ret = pme_ctx_scan(&a_scan_ctx->base_ctx, 0, &fd,
			PME_SCAN_ARGS(PME_CMD_SCAN_FLUSH | PME_CMD_SCAN_SR |
				PME_CMD_SCAN_E, 0, 0xff00), &token);
	LOG(("st: Response scan %d\r\n", ret));
	wait_for_completion(&scan_comp);
	LOG(("st: Status is fd.status %x\r\n",
			a_scan_ctx->result_fd.status));
	status = pme_fd_res_status(&a_scan_ctx->result_fd);
	LOG(("Scan status %x\r\n",status));
	/* Check the response...expect truncation bit to be set */
	if (!(pme_fd_res_flags(&a_scan_ctx->result_fd) & PME_STATUS_TRUNCATED)) {
		pr_info("st: Scan result failed...expected trunc\r\n");
		BUG_ON(1);
	}
	LOG(("st: Simple scan test Passed\r\n"));

	/* read back flow settings */
	ctoken.done = 0;
	ret = pme_ctx_ctrl_read_flow(&a_scan_ctx->base_ctx, PME_CMD_FCW_ALL,
					&rb_flow, &ctoken.token);
	BUG_ON(ret);
	wait_event(NULL, ctoken.done == 1);
	/* Check for error */
	status = pme_fd_res_status(&ctoken.result_fd);
	if (status) {
		pr_info("Read Flow failed %d\r\n", status);
		BUG_ON(1);
	}
#ifdef CONFIG_VERBOSE
	pr_info("st: read Flow Context;\r\n");
	hexdump(&rb_flow, sizeof(rb_flow));
#endif

	ctoken.done = 0;
	ret = pme_ctx_disable(&a_scan_ctx->base_ctx, 0, &ctoken.token);
	BUG_ON(ret < 1);
	wait_event(NULL, ctoken.done == 1);
	pme_ctx_finish(&a_scan_ctx->base_ctx);

	LOG(("st: Scan Test Passed\r\n"));
	loop++;
	progress++;
	if ((progress == PROGRESS) || (loop == NUM_LOOPS)) {
		pr_info("Progress: %d\n", loop);
		progress = 0;
	}
	if (loop < NUM_LOOPS)
		goto loop_start;
	free(sg_table);
	free(scan_data);
	free(result_data);
	free(scan_result_direct_mode_inc_mode);
	free(fl_ctx_exp);
	return 0;
}

